# Generated by Django 2.0.4 on 2018-05-03 09:12

import datetime

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("laws", "0009_auto_20180430_1225"),
    ]

    operations = [
        migrations.AlterField(
            model_name="law",
            name="book",
            field=models.ForeignKey(
                help_text="The book this law belongs to",
                on_delete=django.db.models.deletion.CASCADE,
                to="laws.LawBook",
            ),
        ),
        migrations.AlterField(
            model_name="law",
            name="content",
            field=models.TextField(
                blank=True, help_text="Law content with HTML tags and reference markers"
            ),
        ),
        migrations.AlterField(
            model_name="law",
            name="created_date",
            field=models.DateTimeField(
                auto_now_add=True, help_text="Date of creation of this database entry"
            ),
        ),
        migrations.AlterField(
            model_name="law",
            name="doknr",
            field=models.CharField(
                blank=True,
                help_text="Document number as in XML source",
                max_length=200,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="law",
            name="footnotes",
            field=models.TextField(
                blank=True, help_text="Footnotes as JSON array", null=True
            ),
        ),
        migrations.AlterField(
            model_name="law",
            name="order",
            field=models.PositiveSmallIntegerField(
                default=0, help_text="Order within law book"
            ),
        ),
        migrations.AlterField(
            model_name="law",
            name="previous",
            field=models.ForeignKey(
                help_text="Points to previous law based on order value",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="previous_law",
                to="laws.Law",
            ),
        ),
        migrations.AlterField(
            model_name="law",
            name="slug",
            field=models.SlugField(help_text="Slug based on section", max_length=200),
        ),
        migrations.AlterField(
            model_name="law",
            name="text",
            field=models.TextField(blank=True, help_text="Plain text for searching"),
        ),
        migrations.AlterField(
            model_name="law",
            name="title",
            field=models.CharField(
                default="", help_text="Verbose title of law", max_length=200
            ),
        ),
        migrations.AlterField(
            model_name="law",
            name="updated_date",
            field=models.DateTimeField(
                auto_now=True, help_text="Last change of database entry"
            ),
        ),
        migrations.AlterField(
            model_name="lawbook",
            name="code",
            field=models.CharField(
                default="unknown",
                help_text="Book code (usually short title)",
                max_length=100,
            ),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name="lawbook",
            name="latest",
            field=models.BooleanField(
                default=True,
                help_text="Is true if this is the latest revision of this book",
            ),
        ),
        migrations.AlterField(
            model_name="lawbook",
            name="order",
            field=models.PositiveSmallIntegerField(
                db_index=True,
                default=0,
                help_text="Indicates importance of this law book (used to order books in front end)",
            ),
        ),
        migrations.AlterField(
            model_name="lawbook",
            name="revision_date",
            field=models.DateField(
                default=datetime.date(1990, 1, 1), help_text="Date of revision"
            ),
        ),
        migrations.AlterField(
            model_name="lawbook",
            name="slug",
            field=models.SlugField(help_text="Slugified book code", max_length=200),
        ),
        migrations.AlterField(
            model_name="lawbook",
            name="title",
            field=models.CharField(
                default="Untitled book",
                help_text="Full title of the book",
                max_length=250,
            ),
        ),
        migrations.AlterUniqueTogether(
            name="lawbook",
            unique_together={("slug", "revision_date")},
        ),
    ]



from django.core.management import call_command
from django.test import TestCase, tag


@tag("commands")
class HomepageCommandsTestCase(TestCase):
    def test_dump_api_data(self):
        call_command("dump_api_data", *["api_dump_dir"], **{"override": True})

    def test_render_html_pages(self):
        call_command("render_html_pages", *[], **{})



from unittest.mock import MagicMock, patch

from django.core.cache import cache
from django.http import QueryDict
from django.test import (
    LiveServerTestCase,
    RequestFactory,
    TestCase,
    override_settings,
    tag,
)
from django.urls import reverse

from oldp.apps.search.views import (
    CustomSearchView,
    _get_autocomplete_cache_key,
)
from oldp.utils.test_utils import ElasticsearchTestMixin, es_test


@override_settings(
    STORAGES={
        "staticfiles": {
            "BACKEND": "django.contrib.staticfiles.storage.StaticFilesStorage",
        }
    }
)
@tag("views")
class SearchViewsTestCase(ElasticsearchTestMixin, LiveServerTestCase):
    """Test search views with Elasticsearch (uses mock backend by default)."""

    fixtures = [
        "locations/countries.json",
        "locations/states.json",
        "locations/cities.json",
        "courts/courts.json",
        "cases/cases.json",
    ]

    def setUp(self):
        super().setUp()
        # Index fixture data into mock backend
        self.index_fixtures()

    def tearDown(self):
        pass

    def test_facet_url(self):
        view = CustomSearchView()
        # view.get_search_facets()

        assert view

    def get_search_response(self, query_set):
        qs = QueryDict("", mutable=True)
        qs.update(query_set)

        return self.client.get(reverse("haystack_search") + "?" + qs.urlencode())

    @es_test
    def test_search(self):
        res = self.get_search_response(
            {
                "q": "2 aktg",
            }
        )

        self.assertEqual(200, res.status_code)

    @es_test
    def test_search_with_facets(self):
        res = self.get_search_response(
            {
                "q": "2 aktg",
                "selected_facets": "facet_model_name_exact:Case",
            }
        )

        self.assertEqual(200, res.status_code)

    @es_test
    def test_search_from_unassigned_ref(self):
        res = self.get_search_response({"q": "test", "from": "ref"})
        self.assertEqual(200, res.status_code)


def _make_mock_sqs():
    """Create a MagicMock that behaves like an empty SearchQuerySet."""
    mock_sqs = MagicMock()
    mock_sqs.__len__ = MagicMock(return_value=0)
    mock_sqs.__iter__ = MagicMock(return_value=iter([]))
    mock_sqs.__getitem__ = MagicMock(return_value=[])
    mock_sqs.count.return_value = 0
    mock_sqs.facet_counts.return_value = {
        "fields": {},
        "dates": {},
        "queries": {},
    }
    return mock_sqs


@override_settings(
    COMPRESS_ENABLED=False,
    CACHES={
        "default": {
            "BACKEND": "django.core.cache.backends.locmem.LocMemCache",
            "LOCATION": "mocked-search-views-cache",
        }
    },
    STORAGES={
        "staticfiles": {
            "BACKEND": "django.contrib.staticfiles.storage.StaticFilesStorage",
        }
    },
)
@tag("views")
class MockedSearchViewsTestCase(TestCase):
    """Test search views with mocked search backend (no Elasticsearch required)."""

    def get_search_response(self, query_set):
        qs = QueryDict("", mutable=True)
        qs.update(query_set)
        return self.client.get(reverse("haystack_search") + "?" + qs.urlencode())

    def test_facet_url(self):
        view = CustomSearchView()
        assert view

    @patch(
        "oldp.apps.search.views.CustomSearchForm.search", return_value=_make_mock_sqs()
    )
    def test_search(self, mock_search):
        res = self.get_search_response({"q": "2 aktg"})
        self.assertEqual(200, res.status_code)

    @patch(
        "oldp.apps.search.views.CustomSearchForm.search", return_value=_make_mock_sqs()
    )
    def test_search_with_facets(self, mock_search):
        res = self.get_search_response(
            {
                "q": "2 aktg",
                "selected_facets": "facet_model_name_exact:Case",
            }
        )
        self.assertEqual(200, res.status_code)

    @patch(
        "oldp.apps.search.views.CustomSearchForm.search", return_value=_make_mock_sqs()
    )
    def test_search_from_unassigned_ref(self, mock_search):
        with self.assertLogs("oldp.apps.search.views", level="DEBUG") as cm:
            res = self.get_search_response({"q": "test", "from": "ref"})
        self.assertEqual(200, res.status_code)
        self.assertTrue(any("from=ref" in msg for msg in cm.output))

    @override_settings(
        CACHES={
            "default": {
                "BACKEND": "django.core.cache.backends.locmem.LocMemCache",
                "LOCATION": "search-test-cache",
            }
        },
        STORAGES={
            "staticfiles": {
                "BACKEND": "django.contrib.staticfiles.storage.StaticFilesStorage",
            }
        },
    )
    @patch(
        "oldp.apps.search.views.CustomSearchForm.search", return_value=_make_mock_sqs()
    )
    def test_search_response_is_cached(self, mock_search):
        """Test that search responses are cached by cache_per_role."""
        cache.clear()
        self.get_search_response({"q": "cached query"})
        self.get_search_response({"q": "cached query"})
        mock_search.assert_called_once()  # Second request served from cache
        cache.clear()

    @override_settings(
        CACHES={
            "default": {
                "BACKEND": "django.core.cache.backends.locmem.LocMemCache",
                "LOCATION": "search-facet-test-cache",
            }
        }
    )
    @patch(
        "oldp.apps.search.views.CustomSearchView._build_search_facets", return_value={}
    )
    @patch(
        "oldp.apps.search.views.CustomSearchForm.search", return_value=_make_mock_sqs()
    )
    def test_search_facets_are_cached(self, mock_search, mock_build_facets):
        cache.clear()
        view = CustomSearchView()
        request = RequestFactory().get("/search/?q=test")
        view.request = request
        context = {"facets": {"fields": {}, "dates": {}, "queries": {}}}

        view.get_search_facets(context)
        view.get_search_facets(context)

        mock_build_facets.assert_called_once()
        cache.clear()

    @override_settings(
        CACHES={
            "default": {
                "BACKEND": "django.core.cache.backends.locmem.LocMemCache",
                "LOCATION": "autocomplete-test-cache",
            }
        }
    )
    @patch("oldp.apps.search.views.SearchQuerySet")
    def test_autocomplete_cache_normalizes_query(self, mock_sqs_cls):
        mock_chain = MagicMock()
        mock_chain.__getitem__.return_value = [MagicMock(title="Grundgesetz")]
        mock_sqs = MagicMock()
        mock_sqs.autocomplete.return_value = mock_chain
        mock_sqs_cls.return_value = mock_sqs

        cache.clear()
        res1 = self.client.get("/search/autocomplete?q=%20GG%20")
        res2 = self.client.get("/search/autocomplete?q=gg")

        self.assertEqual(200, res1.status_code)
        self.assertEqual(200, res2.status_code)
        mock_sqs.autocomplete.assert_called_once_with(title="GG")
        mock_sqs_cls.assert_called_once()
        cache.clear()

    @patch("oldp.apps.search.views.SearchQuerySet")
    def test_autocomplete_blank_query_short_circuits(self, mock_sqs_cls):
        res = self.client.get("/search/autocomplete?q=%20%20")

        self.assertEqual(200, res.status_code)
        self.assertJSONEqual(res.content, {"results": []})
        mock_sqs_cls.assert_not_called()

    def test_autocomplete_cache_key_varies_by_host_and_language(self):
        rf = RequestFactory()
        req1 = rf.get("/search/autocomplete?q=gg", HTTP_HOST="example-a.test")
        req1.LANGUAGE_CODE = "en"
        req2 = rf.get("/search/autocomplete?q=GG", HTTP_HOST="example-a.test")
        req2.LANGUAGE_CODE = "en"
        req3 = rf.get("/search/autocomplete?q=gg", HTTP_HOST="example-b.test")
        req3.LANGUAGE_CODE = "en"
        req4 = rf.get("/search/autocomplete?q=gg", HTTP_HOST="example-a.test")
        req4.LANGUAGE_CODE = "de"

        key1 = _get_autocomplete_cache_key(req1, "gg")
        key2 = _get_autocomplete_cache_key(req2, "GG")
        key3 = _get_autocomplete_cache_key(req3, "gg")
        key4 = _get_autocomplete_cache_key(req4, "gg")

        self.assertEqual(key1, key2)  # normalization is case-insensitive for cache key
        self.assertNotEqual(key1, key3)
        self.assertNotEqual(key1, key4)



import datetime
import hashlib
import logging

from django.conf import settings
from django.core.cache import cache
from django.http import JsonResponse
from django.utils.translation import gettext
from django.utils.translation import gettext_lazy as _
from haystack.forms import FacetedSearchForm
from haystack.generic_views import FacetedSearchView
from haystack.query import SearchQuerySet

from oldp.apps.search.utils import is_search_backend_error
from oldp.utils.limited_paginator import LimitedPaginator

logger = logging.getLogger(__name__)


def _normalize_autocomplete_query(query: str) -> str:
    return (query or "").strip()


def _get_autocomplete_cache_key(request, query: str) -> str:
    normalized = _normalize_autocomplete_query(query)
    normalized_key_query = normalized.lower()
    try:
        host = request.get_host()
    except Exception:
        host = request.META.get("HTTP_HOST", "unknown")
    lang = getattr(request, "LANGUAGE_CODE", None) or "default"
    cache_basis = f"{host}|{lang}|{normalized_key_query}"
    digest = hashlib.md5(cache_basis.encode("utf-8")).hexdigest()
    return f"autocomplete_v2_{digest}"


class CustomSearchForm(FacetedSearchForm):
    """Our custom search form for facet search with haystack"""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def search(self):
        # First, store the SearchQuerySet received from other processing.
        sqs = super().search()

        if not self.is_valid():
            return self.no_query_found()

        # Custom date range filter
        # TODO can this be done with native-haystack?
        if "start_date" in self.data:
            try:
                sqs = sqs.filter(
                    date__gte=datetime.datetime.strptime(
                        self.data.get("start_date"), "%Y-%m-%d"
                    )
                )
            except ValueError:
                logger.error("Invalid start_date")

        if "end_date" in self.data:
            try:
                sqs = sqs.filter(
                    date__lte=datetime.datetime.strptime(
                        self.data.get("end_date"), "%Y-%m-%d"
                    )
                )
            except ValueError:
                logger.error("Invalid end_date")

        return sqs


class CustomSearchView(FacetedSearchView):
    """Custom search view for haystack."""

    form_class = CustomSearchForm
    paginator_class = LimitedPaginator
    paginate_by = settings.PAGINATE_BY
    facet_fields = [
        "facet_model_name",
        # Law facets
        "book_code",
        # Case facets
        "decision_type",
        "court",
        "court_jurisdiction",
        "court_level_of_appeal",
        "date",
    ]

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def get_queryset(self):
        qs = super().get_queryset()
        qs = qs.highlight()
        qs = qs.date_facet(
            "date",
            start_date=datetime.date(2009, 6, 7),
            end_date=datetime.date.today(),
            gap_by="year",
            # gap_amount=1,
        )
        return qs

    def _get_search_facets_cache_key(self):
        try:
            host = self.request.get_host()
        except Exception:
            host = self.request.META.get("HTTP_HOST", "unknown")
        lang = getattr(self.request, "LANGUAGE_CODE", None) or "default"
        cache_basis = f"{host}|{lang}|{self.request.get_full_path()}"
        digest = hashlib.md5(cache_basis.encode("utf-8")).hexdigest()
        return f"search_facets_v1_{digest}"

    def _build_search_facets(self, context):
        """Convert haystack facets to make it easier to build a nice facet sidebar"""
        selected_facets = {}
        qs_facets = self.request.GET.getlist("selected_facets")

        for qp in qs_facets:
            tmp = qp.split("_exact:")

            if len(tmp) == 2:
                selected_facets[tmp[0]] = tmp[1]

            else:
                tmp2 = qp.split(":")

                if len(tmp2) == 2:
                    selected_facets[tmp2[0]] = tmp2[1]

        facets = {}

        if "fields" in context["facets"]:
            for facet_name in context["facets"]["fields"]:
                # if self.request.GET[facet_name]
                facets[facet_name] = {
                    "name": facet_name,
                    "selected": facet_name in selected_facets,
                    "choices": [],
                }

                # All choices
                for facet_choices in context["facets"]["fields"][facet_name]:
                    value, count = facet_choices
                    selected = (
                        facet_name in selected_facets
                        and selected_facets[facet_name] == value
                    )
                    url_param = facet_name + "_exact:%s" % value
                    qs = self.request.GET.copy()

                    if selected:
                        # Remove current facet from url
                        _selected_facets = []
                        for f in qs.getlist("selected_facets"):
                            if f != url_param:
                                _selected_facets.append(f)

                        del qs["selected_facets"]
                        qs.setlist("selected_facets", _selected_facets)

                    else:
                        # Add facet to url
                        qs.update({"selected_facets": url_param})

                    # Filter links should not have pagination
                    if "page" in qs:
                        del qs["page"]

                    if facet_name == "facet_model_name":
                        value = gettext(value)

                    facets[facet_name]["choices"].append(
                        {
                            "facet_name": facet_name,
                            "value": value,
                            "count": count,
                            "selected": selected,
                            "url": "?" + qs.urlencode(),
                        }
                    )

                # Remove empty facets
                if not facets[facet_name]["choices"]:
                    del facets[facet_name]

        return facets

    def get_search_facets(self, context):
        cache_key = self._get_search_facets_cache_key()
        cached_facets = cache.get(cache_key)
        if cached_facets is not None:
            return cached_facets

        facets = self._build_search_facets(context)
        cache.set(cache_key, facets, settings.CACHE_TTL)
        return facets

    def get_context_data(self, *args, **kwargs):
        try:
            context = super().get_context_data(**kwargs)
        except Exception as exc:
            if is_search_backend_error(exc):
                logger.error("Search backend unavailable: %s", exc)
                context = {"query": self.request.GET.get("q", ""), "facets": {}}
                context.update(
                    {
                        "title": _("Search"),
                        "search_error": _(
                            "Search is currently unavailable. Please try again later."
                        ),
                        "search_facets": {},
                    }
                )
                return context
            raise

        search_from = self.request.GET.get("from")
        selected_facets = self.request.GET.getlist("selected_facets")
        logger.debug(
            "Search query: %s (from=%s, facets=%s)",
            context["query"],
            search_from,
            selected_facets or None,
        )

        # TODO data facets are disabled for now
        # date_facets = {}

        # if 'dates' in context['facets'] and 'date' in context['facets']['dates']:  # we assume that dates are already sorted
        #     dates = context['facets']['dates']['date']

        #     if len(dates) > 1:
        #         fmt = '%Y-%m-%d'
        #         date_facets = {
        #             'start_date': dates[0][0].strftime(fmt),
        #             'end_date': dates[-1][0].strftime(fmt),
        #             'items': [{'date': date.strftime(fmt), 'count': count} for date, count in dates],
        #         }

        context.update(
            {
                "title": _("Search") + " " + context["query"][:30],
                "search_facets": self.get_search_facets(context),
                # 'date_facets': date_facets,
            }
        )

        return context


def autocomplete_view(request):
    """Stub for auto-complete feature(title for all objects missing)"""
    suggestions_limit = 5
    query = _normalize_autocomplete_query(request.GET.get("q", ""))

    if not query:
        return JsonResponse({"results": []})

    cache_key = _get_autocomplete_cache_key(request, query)
    cached = cache.get(cache_key)
    if cached is not None:
        return JsonResponse({"results": cached})

    try:
        sqs = SearchQuerySet().autocomplete(title=query)[:suggestions_limit]
        suggestions = [result.title for result in sqs]
    except Exception as e:
        logger.error("Autocomplete search failed for query '%s': %s", query, str(e))
        if is_search_backend_error(e):
            return JsonResponse(
                {"error": "Search is currently unavailable."},
                status=503,
            )
        return JsonResponse({"results": []})

    cache.set(cache_key, suggestions, settings.CACHE_TTL)
    return JsonResponse({"results": suggestions})



import os

from django.conf import settings
from django.core.management.base import BaseCommand

from oldp.apps.cases.processing.case_processor import (
    CaseInputHandlerDB,
    CaseInputHandlerFS,
    CaseProcessor,
)


class Command(BaseCommand):
    help = "Processes cases from FS or DB with different processing steps (extract refs, ...)"
    indexer = CaseProcessor()

    def add_arguments(self, parser):
        self.indexer.set_parser_arguments(parser)
        CaseInputHandlerDB.set_parser_arguments(parser)

        parser.add_argument(
            "--input",
            nargs="+",
            type=str,
            default=os.path.join(settings.BASE_DIR, "workingdir", "cases"),
        )
        parser.add_argument(
            "--input-handler",
            type=str,
            default="db",
            help="Read input from this source (file system or database)",
            choices=["db", "fs"],
        )

        parser.add_argument("--max-lines", type=int, default=-1)

        parser.add_argument(
            "--source",
            type=str,
            default="serializer",
            help="When reading from FS process files differently (serializer)",
        )

        parser.add_argument(
            "--empty", action="store_true", default=False, help="Empty existing index"
        )

    def handle(self, *args, **options):
        self.indexer.set_options(options)

        # Define input
        if options["input_handler"] == "fs":
            if options["source"] == "serializer":
                handler = CaseInputHandlerFS(
                    limit=options["limit"],
                    start=options["start"],
                    selector=options["input"],
                )
            else:
                raise ValueError("Mode not supported. Use openjur or serializer.")

        elif options["input_handler"] == "db":
            handler = CaseInputHandlerDB(
                limit=options["limit"],
                start=options["start"],
                filter_qs=options["filter"],
                exclude_qs=options["exclude"],
                order_by=options["order_by"],
                per_page=options["per_page"],
            )

        else:
            raise ValueError("Unsupported input handler: %s" % options["input_handler"])

        self.indexer.set_input_handler(handler)

        # Prepare processing steps
        self.indexer.set_processing_steps(options["step"])

        if options["empty"]:
            self.indexer.empty_content()

        # Do processing
        self.indexer.process()
        self.indexer.log_stats()



"""http://gg.docpatch.org/

http://gg.docpatch.org/grundgesetz-dev/out/brd_grundgesetz_59_2012-07-16.md

1. create law book (set icon=Art.
2. fetch version links from gg.docpatch
    a) download version md
    b) create law instance (extract refs)
    c) assign refs

TODO Alternative source:
    - Scrape from MD https://github.com/c3e/grundgesetz

"""

import logging

import pypandoc
import requests
from django.core.management import BaseCommand
from lxml import etree
from lxml.etree import tostring
from slugify import slugify

from oldp.apps.laws.models import Law, LawBook

logger = logging.getLogger(__name__)

# os.environ.setdefault('PYPANDOC_PANDOC', '/home/x/whatever/pandoc')


class Command(BaseCommand):
    help = 'Import law book "Grundgesetz" (German basic law)'

    def __init__(self):
        super(Command, self).__init__()

    def add_arguments(self, parser):
        parser.add_argument("--limit", type=int, default=0)
        parser.add_argument(
            "--empty",
            action="store_true",
            default=False,
            help="Delete existing revisions",
        )

    def handle_law_from_xml(self, book, book_xml) -> LawBook:
        previous_law = None
        law_order = 1

        # Parse XML tree
        tree = etree.fromstring(book_xml)

        for sect in tree.xpath("sect1"):
            section_title = sect.xpath("title/text()")[0]
            logger.debug("Section: %s" % section_title)

            # if section_title == 'Grundgesetz für die Bundesrepublik Deutschland':
            #     continue

            book.add_section(from_order=law_order, title=section_title.strip())

            for law_key, law_raw in enumerate(sect.xpath("sect2")):
                law_title = law_raw.xpath("title")[0]
                law_title.getparent().remove(law_title)

                # law_docbook = tostring(law_raw).decode('utf-8')
                law_docbook = "\n".join(
                    tostring(x).decode("utf-8") for x in law_raw.iterchildren()
                )
                law_text = pypandoc.convert_text(law_docbook, "html", format="docbook")
                law_section = tostring(law_title, method="text").decode("utf-8").strip()

                law = Law(
                    book=book,
                    title="",
                    section=law_section,
                    slug=slugify(law_section),
                    content=law_text,
                    previous=previous_law,
                    order=law_order,
                )
                law.save()
                law_order += 1
                previous_law = law

        return book

    def handle(self, *args, **options):
        book_slug = "gg"

        # Fetch meta information
        meta = requests.get(
            url="http://gg.docpatch.org/grundgesetz-dev/etc/meta.json"
        ).json()

        logger.info("Import GG: %s" % meta["coverage"])

        # Delete old GG
        if options["empty"]:
            logger.debug("Deleting old law books (slug=gg only)")
            LawBook.objects.filter(slug=book_slug).delete()

        revs = list(reversed(list(meta["revisions"])))

        if options["limit"] > 0:
            revs = revs[: options["limit"]]

        # Iterate over revisions
        for key, rev in enumerate(revs):
            # Exists already?
            if not LawBook.objects.filter(
                slug=book_slug, revision_date=rev["announced"]
            ).exists():
                book = LawBook(
                    code="Grundgesetz",
                    title="Grundgesetz für die Bundesrepublik Deutschland",
                    slug=book_slug,
                    revision_date=rev["announced"],
                    changelog='[{"type":"Stand", "text": "%s"}]' % rev["title"],
                    latest=key == 0,
                )
                book.save()
                logger.debug("Law book saved: %s" % book)

                # Download details
                # brd_grundgesetz_59_2012-07-16.html
                book_url = (
                    "http://gg.docpatch.org/grundgesetz-dev/out/brd_grundgesetz_%i_%s.db"
                    % (len(meta["revisions"]) - key - 1, rev["announced"])
                )

                logger.debug("Downloading from: %s" % book_url)
                book_xml = requests.get(url=book_url).text.encode("utf-8")

                book = self.handle_law_from_xml(book, book_xml)
                book.save()  # Save sections



import logging
import re

from django.conf import settings

from oldp.apps.courts.apps import CourtLocationLevel
from oldp.apps.courts.models import Court
from oldp.apps.courts.processing import CourtProcessingStep

logger = logging.getLogger(__name__)


class ProcessingStep(CourtProcessingStep):
    """Aliases should make it easier to find matching courts (e.g. Court.objects.get(aliases__contains=...))"""

    description = "Set aliases for courts"

    def combine_type_location(self, types, location):
        """Combine type and location in both orders (AG Aachen + Aachen AG)"""
        for t in types:
            yield t + " " + location
            yield location + " " + t

    def process(self, court: Court) -> Court:
        """Generates all possible aliases for court names

        AG Aachen
        Aachen AG
        Aachener AG
        """
        aliases = [
            # Name is always as well an alias
            court.name
        ]

        if court.court_type is None:
            logger.warning("No court type: %s" % court)
            return court

        type_info = settings.COURT_TYPES.get_type(court.court_type)
        location_levels = type_info["levels"]

        type_aliases = [
            court.court_type,
            type_info["name"],
        ]

        if CourtLocationLevel.CITY in location_levels:
            # Frankfurt (Oder)
            # ... an der Oder
            # Frankfurt am Main
            loc_name = court.city.name

            aliases.extend(self.combine_type_location(type_aliases, loc_name))

            for match in re.finditer(
                r"\s(a\.d\.|an der|am|im|unter|in der)\s(.*?)$", loc_name
            ):
                # Frankfurt an der Oder -> Frankfurt (Oder)
                loc_name_x = "%s (%s)" % (loc_name[: match.start(0)], match.group(2))

                aliases.extend(self.combine_type_location(type_aliases, loc_name_x))

        if CourtLocationLevel.STATE in location_levels:
            # Add variations, e.g. Hamburg_er, Holstein_isches
            loc_name = court.state.name

            aliases.extend(self.combine_type_location(type_aliases, loc_name))

            for t in type_aliases:
                for v in ["es", "er", "isches"]:
                    aliases.append(loc_name + v + " " + t)

        if CourtLocationLevel.COUNTRY in location_levels:
            # TODO Handle federal courts (BGH, ...)
            pass

        # Set as list
        court.aliases = Court.ALIAS_SEPARATOR.join(aliases)

        return court



# Generated by Django 2.1.2 on 2018-10-30 12:09

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("topics", "0001_initial"),
        ("laws", "0015_remove_law_text"),
    ]

    operations = [
        migrations.AddField(
            model_name="lawbook",
            name="topics",
            field=models.ManyToManyField(
                help_text="Topics that are covered by this content", to="topics.Topic"
            ),
        ),
    ]



import logging

from django.core.management import BaseCommand

from oldp.apps.laws.models import LawBook

LAW_BOOK_ORDER = {
    "bgb": 10,
    "gg": 10,
    "bafog": 9,
    "zpo": 9,
    "stgb": 9,
    "stpo": 9,
    "hgb": 9,
    "inso": 8,
    "weg": 8,
    "baugb": 8,
    "agg": 8,
    "urhg": 8,
    "dsgvo": 8,
}

# Get an instance of a logger
logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = "Assign predefined order values to law books based on slug"

    def __init__(self):
        super(Command, self).__init__()

    def handle(self, *args, **options):
        order_mapping = LAW_BOOK_ORDER

        for book_slug in order_mapping:
            updates = LawBook.objects.filter(slug=book_slug).update(
                order=order_mapping[book_slug]
            )

            if updates > 0:
                logger.debug("Updated %s" % book_slug)
            else:
                logger.debug("Does not exist: %s" % book_slug)

        logger.info("done")



"""Unit tests for the cache_per_user and cache_per_role decorators.

Tests cover:
- Caching for anonymous users
- Caching for authenticated users
- POST request handling
- TTL and prefix options
- Role-based caching (anon/auth/staff)
"""

import logging
from unittest.mock import MagicMock, patch

from django.core.cache import cache
from django.http import HttpResponse
from django.template.response import TemplateResponse
from django.test import RequestFactory, TestCase, override_settings, tag

from oldp.utils.cache_per_user import cache_per_role, cache_per_user

logger = logging.getLogger(__name__)


# Use locmem cache for testing since TestConfiguration uses DummyCache
CACHES_OVERRIDE = {
    "default": {
        "BACKEND": "django.core.cache.backends.locmem.LocMemCache",
        "LOCATION": "unique-snowflake",
    }
}


@tag("utils", "cache")
@override_settings(CACHES=CACHES_OVERRIDE)
class CachePerUserTestCase(TestCase):
    """Tests for the cache_per_user decorator."""

    def setUp(self):
        self.factory = RequestFactory()
        cache.clear()

    def tearDown(self):
        cache.clear()

    def test_caches_anonymous_user_response(self):
        """Test that responses for anonymous users are cached."""
        call_count = {"count": 0}

        @cache_per_user(ttl=60)
        def view_func(request):
            call_count["count"] += 1
            return HttpResponse("Hello")

        request = self.factory.get("/test/")
        request.user = MagicMock()
        request.user.is_authenticated = False

        # First call
        response1 = view_func(request)
        self.assertEqual(call_count["count"], 1)

        # Second call - should use cache
        response2 = view_func(request)
        self.assertEqual(call_count["count"], 1)  # Still 1, used cache

        self.assertEqual(response1.content, response2.content)

    def test_caches_authenticated_user_separately(self):
        """Test that authenticated users have separate cache entries."""
        call_count = {"count": 0}

        @cache_per_user(ttl=60)
        def view_func(request):
            call_count["count"] += 1
            return HttpResponse(f"Hello user {request.user.id}")

        request1 = self.factory.get("/test/")
        request1.user = MagicMock()
        request1.user.is_authenticated = True
        request1.user.id = 1

        request2 = self.factory.get("/test/")
        request2.user = MagicMock()
        request2.user.is_authenticated = True
        request2.user.id = 2

        # Calls for different users should not share cache
        view_func(request1)
        self.assertEqual(call_count["count"], 1)

        view_func(request2)
        self.assertEqual(call_count["count"], 2)

    def test_anonymous_users_share_cache(self):
        """Test that all anonymous users share the same cache entry."""
        call_count = {"count": 0}

        @cache_per_user(ttl=60)
        def view_func(request):
            call_count["count"] += 1
            return HttpResponse("Anonymous content")

        request1 = self.factory.get("/test/")
        request1.user = MagicMock()
        request1.user.is_authenticated = False

        request2 = self.factory.get("/test/")
        request2.user = MagicMock()
        request2.user.is_authenticated = False

        # Both anonymous users should share cache
        view_func(request1)
        view_func(request2)
        self.assertEqual(call_count["count"], 1)

    def test_post_request_not_cached_by_default(self):
        """Test that POST requests are not cached by default."""
        call_count = {"count": 0}

        @cache_per_user(ttl=60)
        def view_func(request):
            call_count["count"] += 1
            return HttpResponse("Response")

        request = self.factory.post("/test/")
        request.user = MagicMock()
        request.user.is_authenticated = False

        view_func(request)
        view_func(request)
        self.assertEqual(call_count["count"], 2)  # Called twice, no caching

    def test_post_request_cached_when_enabled(self):
        """Test that POST requests are cached when cache_post=True."""
        call_count = {"count": 0}

        @cache_per_user(ttl=60, cache_post=True)
        def view_func(request):
            call_count["count"] += 1
            return HttpResponse("Response")

        request = self.factory.post("/test/")
        request.user = MagicMock()
        request.user.is_authenticated = False

        view_func(request)
        view_func(request)
        self.assertEqual(call_count["count"], 1)  # Only called once, cached

    def test_custom_prefix(self):
        """Test that custom prefix is used in cache key."""
        call_count = {"count": 0}

        @cache_per_user(ttl=60, prefix="my_custom_prefix")
        def view_func(request):
            call_count["count"] += 1
            return HttpResponse("Response")

        request = self.factory.get("/test/")
        request.user = MagicMock()
        request.user.is_authenticated = False

        view_func(request)

        # Check that cache key uses prefix
        cache_key = "my_custom_prefix_anonymous"
        self.assertIsNotNone(cache.get(cache_key))

    def test_different_paths_have_different_cache(self):
        """Test that different paths have different cache entries."""
        call_count = {"count": 0}

        @cache_per_user(ttl=60)
        def view_func(request):
            call_count["count"] += 1
            return HttpResponse("Response")

        request1 = self.factory.get("/path1/")
        request1.user = MagicMock()
        request1.user.is_authenticated = False

        request2 = self.factory.get("/path2/")
        request2.user = MagicMock()
        request2.user.is_authenticated = False

        view_func(request1)
        view_func(request2)
        self.assertEqual(call_count["count"], 2)  # Different paths, different cache

    def test_same_path_different_query_params(self):
        """Test that different query params create different cache entries."""
        call_count = {"count": 0}

        @cache_per_user(ttl=60)
        def view_func(request):
            call_count["count"] += 1
            return HttpResponse("Response")

        request1 = self.factory.get("/test/?page=1")
        request1.user = MagicMock()
        request1.user.is_authenticated = False

        request2 = self.factory.get("/test/?page=2")
        request2.user = MagicMock()
        request2.user.is_authenticated = False

        view_func(request1)
        view_func(request2)
        self.assertEqual(call_count["count"], 2)  # Different query params

    def test_renders_template_response(self):
        """Test that TemplateResponse is rendered before caching."""
        call_count = {"count": 0}

        @cache_per_user(ttl=60)
        def view_func(request):
            call_count["count"] += 1
            response = TemplateResponse(request, "test_template.html", {})
            return response

        request = self.factory.get("/test/")
        request.user = MagicMock()
        request.user.is_authenticated = False

        with patch.object(TemplateResponse, "render") as mock_render:
            mock_render.return_value = HttpResponse("Rendered content")
            view_func(request)
            mock_render.assert_called_once()

    def test_no_ttl_uses_default_cache_timeout(self):
        """Test that not specifying TTL uses cache default."""
        call_count = {"count": 0}

        @cache_per_user()
        def view_func(request):
            call_count["count"] += 1
            return HttpResponse("Response")

        request = self.factory.get("/test/")
        request.user = MagicMock()
        request.user.is_authenticated = False

        view_func(request)
        view_func(request)
        self.assertEqual(call_count["count"], 1)

    def test_passes_args_to_view(self):
        """Test that args and kwargs are passed to the view function."""

        @cache_per_user(ttl=60)
        def view_func(request, pk, name=None):
            return HttpResponse(f"pk={pk}, name={name}")

        request = self.factory.get("/test/")
        request.user = MagicMock()
        request.user.is_authenticated = False

        response = view_func(request, 42, name="test")

        self.assertIn(b"pk=42", response.content)
        self.assertIn(b"name=test", response.content)


@tag("utils", "cache")
@override_settings(CACHES=CACHES_OVERRIDE)
class CachePerRoleTestCase(TestCase):
    """Tests for the cache_per_role decorator."""

    def setUp(self):
        self.factory = RequestFactory()
        cache.clear()

    def tearDown(self):
        cache.clear()

    def _make_user(self, authenticated=False, is_staff=False, user_id=None):
        user = MagicMock()
        user.is_authenticated = authenticated
        user.is_staff = is_staff
        user.id = user_id
        return user

    def test_caches_anonymous_response(self):
        """Test that anonymous responses are cached under the 'anon' role."""
        call_count = {"count": 0}

        @cache_per_role(ttl=60)
        def view_func(request):
            call_count["count"] += 1
            return HttpResponse("Anon content")

        request = self.factory.get("/test/")
        request.user = self._make_user()

        view_func(request)
        view_func(request)
        self.assertEqual(call_count["count"], 1)

    def test_caches_authenticated_response(self):
        """Test that authenticated non-staff responses are cached under 'auth' role."""
        call_count = {"count": 0}

        @cache_per_role(ttl=60)
        def view_func(request):
            call_count["count"] += 1
            return HttpResponse("Auth content")

        request = self.factory.get("/test/")
        request.user = self._make_user(authenticated=True, user_id=1)

        view_func(request)
        view_func(request)
        self.assertEqual(call_count["count"], 1)

    def test_caches_staff_response(self):
        """Test that staff responses are cached under 'staff' role."""
        call_count = {"count": 0}

        @cache_per_role(ttl=60)
        def view_func(request):
            call_count["count"] += 1
            return HttpResponse("Staff content")

        request = self.factory.get("/test/")
        request.user = self._make_user(authenticated=True, is_staff=True, user_id=1)

        view_func(request)
        view_func(request)
        self.assertEqual(call_count["count"], 1)

    def test_three_roles_get_separate_entries(self):
        """Test that anon, auth, and staff each get separate cache entries."""
        call_count = {"count": 0}

        @cache_per_role(ttl=60)
        def view_func(request):
            call_count["count"] += 1
            return HttpResponse("Content")

        anon_req = self.factory.get("/test/")
        anon_req.user = self._make_user()

        auth_req = self.factory.get("/test/")
        auth_req.user = self._make_user(authenticated=True, user_id=1)

        staff_req = self.factory.get("/test/")
        staff_req.user = self._make_user(authenticated=True, is_staff=True, user_id=2)

        view_func(anon_req)
        view_func(auth_req)
        view_func(staff_req)
        self.assertEqual(call_count["count"], 3)

    def test_two_authenticated_users_share_cache(self):
        """Test that two non-staff authenticated users share the same cache entry."""
        call_count = {"count": 0}

        @cache_per_role(ttl=60)
        def view_func(request):
            call_count["count"] += 1
            return HttpResponse("Shared auth content")

        request1 = self.factory.get("/test/")
        request1.user = self._make_user(authenticated=True, user_id=1)

        request2 = self.factory.get("/test/")
        request2.user = self._make_user(authenticated=True, user_id=2)

        view_func(request1)
        view_func(request2)
        self.assertEqual(call_count["count"], 1)

    def test_two_staff_users_share_cache(self):
        """Test that two staff users share the same cache entry."""
        call_count = {"count": 0}

        @cache_per_role(ttl=60)
        def view_func(request):
            call_count["count"] += 1
            return HttpResponse("Shared staff content")

        request1 = self.factory.get("/test/")
        request1.user = self._make_user(authenticated=True, is_staff=True, user_id=1)

        request2 = self.factory.get("/test/")
        request2.user = self._make_user(authenticated=True, is_staff=True, user_id=2)

        view_func(request1)
        view_func(request2)
        self.assertEqual(call_count["count"], 1)

    def test_post_not_cached_by_default(self):
        """Test that POST requests are not cached by default."""
        call_count = {"count": 0}

        @cache_per_role(ttl=60)
        def view_func(request):
            call_count["count"] += 1
            return HttpResponse("Response")

        request = self.factory.post("/test/")
        request.user = self._make_user()

        view_func(request)
        view_func(request)
        self.assertEqual(call_count["count"], 2)

    def test_post_cached_when_enabled(self):
        """Test that POST requests are cached when cache_post=True."""
        call_count = {"count": 0}

        @cache_per_role(ttl=60, cache_post=True)
        def view_func(request):
            call_count["count"] += 1
            return HttpResponse("Response")

        request = self.factory.post("/test/")
        request.user = self._make_user()

        view_func(request)
        view_func(request)
        self.assertEqual(call_count["count"], 1)

    def test_different_paths_have_different_cache(self):
        """Test that different paths produce different cache entries."""
        call_count = {"count": 0}

        @cache_per_role(ttl=60)
        def view_func(request):
            call_count["count"] += 1
            return HttpResponse("Response")

        request1 = self.factory.get("/path1/")
        request1.user = self._make_user()

        request2 = self.factory.get("/path2/")
        request2.user = self._make_user()

        view_func(request1)
        view_func(request2)
        self.assertEqual(call_count["count"], 2)

    def test_different_query_params_have_different_cache(self):
        """Test that different query params produce different cache entries."""
        call_count = {"count": 0}

        @cache_per_role(ttl=60)
        def view_func(request):
            call_count["count"] += 1
            return HttpResponse("Response")

        request1 = self.factory.get("/test/?page=1")
        request1.user = self._make_user()

        request2 = self.factory.get("/test/?page=2")
        request2.user = self._make_user()

        view_func(request1)
        view_func(request2)
        self.assertEqual(call_count["count"], 2)

    def test_renders_template_response(self):
        """Test that TemplateResponse is rendered before caching."""

        @cache_per_role(ttl=60)
        def view_func(request):
            return TemplateResponse(request, "test_template.html", {})

        request = self.factory.get("/test/")
        request.user = self._make_user()

        with patch.object(TemplateResponse, "render") as mock_render:
            mock_render.return_value = HttpResponse("Rendered content")
            view_func(request)
            mock_render.assert_called_once()

    def test_different_languages_have_different_cache_entries(self):
        call_count = {"count": 0}

        @cache_per_role(ttl=60)
        def view_func(request):
            call_count["count"] += 1
            return HttpResponse("Response")

        request1 = self.factory.get("/test/")
        request1.user = self._make_user()
        request1.LANGUAGE_CODE = "en"

        request2 = self.factory.get("/test/")
        request2.user = self._make_user()
        request2.LANGUAGE_CODE = "de"

        view_func(request1)
        view_func(request2)
        self.assertEqual(call_count["count"], 2)

    @override_settings(
        ALLOWED_HOSTS=["testserver", "en.example.test", "de.example.test"]
    )
    def test_different_hosts_have_different_cache_entries(self):
        call_count = {"count": 0}

        @cache_per_role(ttl=60)
        def view_func(request):
            call_count["count"] += 1
            return HttpResponse("Response")

        request1 = self.factory.get("/test/", HTTP_HOST="en.example.test")
        request1.user = self._make_user()

        request2 = self.factory.get("/test/", HTTP_HOST="de.example.test")
        request2.user = self._make_user()

        view_func(request1)
        view_func(request2)
        self.assertEqual(call_count["count"], 2)

    def test_set_cookie_responses_are_not_cached(self):
        call_count = {"count": 0}

        @cache_per_role(ttl=60)
        def view_func(request):
            call_count["count"] += 1
            response = HttpResponse("Response")
            response.set_cookie("sessionid", "abc")
            return response

        request = self.factory.get("/test/")
        request.user = self._make_user()

        view_func(request)
        view_func(request)
        self.assertEqual(call_count["count"], 2)

    def test_vary_cookie_responses_are_not_cached(self):
        call_count = {"count": 0}

        @cache_per_role(ttl=60)
        def view_func(request):
            call_count["count"] += 1
            response = HttpResponse("Response")
            response["Vary"] = "Cookie"
            return response

        request = self.factory.get("/test/")
        request.user = self._make_user()

        view_func(request)
        view_func(request)
        self.assertEqual(call_count["count"], 2)

    def test_uses_view_cache_prefix(self):
        """Test that cache keys use the view_cache_ prefix for signal handler compatibility."""

        @cache_per_role(ttl=60)
        def view_func(request):
            return HttpResponse("Response")

        request = self.factory.get("/laws/test/")
        request.user = self._make_user()

        view_func(request)

        cache_key = "view_cache_testserver_default_GET_/laws/test/_anon"
        self.assertIsNotNone(cache.get(cache_key))



# -*- coding: utf-8 -*-
import os
from unittest import skip

from django.test import TestCase, tag

from oldp.apps.laws.models import Law
from oldp.apps.laws.processing.law_processor import LawInputHandlerFS, LawProcessor
from oldp.apps.laws.processing.processing_steps.extract_refs import (
    ProcessingStep as ExtractRefsStep,
)

RESOURCE_DIR = os.path.join(os.path.dirname(os.path.realpath(__file__)), "resources")


@tag("processing")
class LawsProcessingTestCase(TestCase):
    fixtures = ["laws/laws.json"]

    def __init__(self, *args, **kwargs):
        super(LawsProcessingTestCase, self).__init__(*args, **kwargs)

    def setUp(self):
        pass

    def tearDown(self):
        # os.remove(self.temp_path)
        pass

    @skip
    def test_extract_refs(self):
        # TODO currently not working
        content = Law.objects.get(book__slug="ublg-1", slug="12")
        # print(content)
        # print(content.get_reference_markers())

        step = ExtractRefsStep()
        processed = step.process(content)

        # print(processed)
        # print(processed.reference_markers)

        self.assertEqual(
            len(processed.get_reference_markers()),
            1,
            "Invalid number of reference markers",
        )
        # self.assertEqual(processed.get_reference_markers()[0].referenced_by, content, 'Invalid reference markers by')
        # self.assertEqual(processed.get_reference_markers()[0].start, 764, 'Invalid reference markers start')
        # self.assertEqual(processed.get_reference_markers()[0].end, 767, 'Invalid reference markers end')

    @skip
    def test_extract_refs_detail(self):
        # TODO currently not working
        law = Law.objects.get(book__slug="ublg-1", slug="12")
        law_processed = ExtractRefsStep().process(law)

        assert law_processed

        # print('Original: %s' % text)
        # print('\nMarker: %s' % ref_text)

        # self.assertEqual(len(law_processed.get_references()), 3, 'Invalid number of reference markers')

        # TODO Reference count?

        # print(refs)

    def test_empty_content(self):
        processor = LawProcessor()
        law_books, laws, markers = processor.empty_content()

        deleted_rows, info = law_books  # Contains other types from foreign keys as well

        self.assertEqual(123, deleted_rows, "Invalid number of rows deleted")

    def test_law_input_handler_fs(self):
        ih = LawInputHandlerFS(
            limit=10, selector=os.path.join(RESOURCE_DIR, "from_bundesgit")
        )
        ih.pre_processed_content = []
        # test get_input
        self.assertEqual(2, len(ih.get_input()), "Invalid number of input files")

        # test handle_input
        ih.handle_input(
            os.path.join(
                RESOURCE_DIR, "from_bundesgit", "s", "stvo_2013", "stvo_2013.xml"
            )
        )
        items = ih.pre_processed_content  # type: List[Law]

        self.assertEqual(61, len(items), "Invalid count")
        self.assertEqual("Grundregeln", items[0].title, "Invalid title")
        self.assertEqual("1", items[0].slug, "Invalid slug")
        self.assertEqual(
            "Straßenverkehrs-Ordnung", items[0].book.title, "Invalid book title"
        )

        ih.pre_processed_content = []
        ih.handle_input(
            os.path.join(RESOURCE_DIR, "from_bundesgit", "b", "baunvo", "baunvo.xml")
        )
        items = ih.pre_processed_content  # type: List[Law]

        self.assertEqual(35, len(items), "Invalid count")
        self.assertEqual("Reine Wohngebiete", items[3].title, "Invalid title")
        self.assertEqual("3", items[3].slug, "Invalid slug")
        self.assertEqual(
            "Verordnung über die bauliche Nutzung der Grundstücke",
            items[3].book.title,
            "Invalid book title",
        )

        # print(items)



from django.urls import re_path

from oldp.apps.accounts import views

# app_name = 'accounts'

urlpatterns = [
    re_path(r"^profile/$", views.profile_view, name="account_profile"),
    re_path(r"^api/$", views.api_view, name="account_api"),
    re_path(r"^api/renew/$", views.api_renew_view, name="account_api_renew"),
    # Multi-token system URLs
    re_path(r"^api/tokens/$", views.api_tokens_list_view, name="account_api_tokens"),
    re_path(
        r"^api/tokens/create/$",
        views.api_token_create_view,
        name="account_api_token_create",
    ),
    re_path(
        r"^api/tokens/(?P<token_id>\d+)/revoke/$",
        views.api_token_revoke_view,
        name="account_api_token_revoke",
    ),
]



import logging

from django.test import TestCase, tag

from oldp.apps.cases.models import Case
from oldp.apps.references.models import (
    CaseReferenceMarker,
    Reference,
    ReferenceFromCase,
)

logger = logging.getLogger(__name__)


@tag("models")
class ReferencesModelsTestCase(TestCase):
    fixtures = [
        "locations/countries.json",
        "locations/states.json",
        "locations/cities.json",
        "courts/courts.json",
        "cases/cases.json",
    ]

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_m2m(self):
        c1 = Case.objects.get(pk=1)
        c2 = Case.objects.get(pk=2)

        r1 = Reference(case=c2, to="case2-1")
        r1.save()

        marker = CaseReferenceMarker(text="foo", referenced_by=c1)
        marker.save()

        ReferenceFromCase.objects.create(reference=r1, marker=marker)
        ReferenceFromCase.objects.create(
            reference=Reference.objects.create(case=c2, to="case2"), marker=marker
        )

        self.assertEqual(
            2, len(marker.references.all()), "Invalid number of references"
        )
        self.assertEqual(
            2,
            Reference.objects.filter(casereferencemarker=marker).count(),
            "Invalid number of references (reverse look-up)",
        )

        self.assertEqual(
            1, r1.casereferencemarker_set.count(), "Reverse set lookup failed"
        )

        self.assertEqual(r1.get_marker(), marker, "Invalid marker")

    def test_get_absolute_url(self):
        c1 = Case.objects.get(pk=1)
        r1 = Reference(case=c1, to="case2-1")

        self.assertEqual(r1.get_absolute_url(), c1.get_absolute_url(), "Invalid url")

    def test_get_absolute_url_unassigned(self):
        c1 = Case.objects.get(pk=1)
        r1 = Reference(to="test ref")
        r1.save()

        marker = CaseReferenceMarker(text="§ 123 BGB", referenced_by=c1)
        marker.save()

        ReferenceFromCase.objects.create(reference=r1, marker=marker)

        url = r1.get_absolute_url()
        self.assertIn("/search/", url)
        self.assertIn("from=ref", url)



# Generated by Django 2.1.2 on 2019-02-18 11:44

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("references", "0008_remove_uuid__change_line_to_int"),
    ]

    operations = [
        migrations.AlterField(
            model_name="reference",
            name="case",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="cases.Case",
            ),
        ),
        migrations.AlterField(
            model_name="reference",
            name="law",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="laws.Law",
            ),
        ),
    ]



try:
    import importlib.metadata

    __version__ = importlib.metadata.version("oldp")
except (importlib.metadata.PackageNotFoundError, ImportError):
    # Fallback version when package is not installed
    __version__ = "0.9.10"
#
# from configurations import importer

# # Enable django-configuration support for PyCharm
# importer.install(check_options=True)



"""Custom admin views for case statistics dashboard."""

from oldp.apps.cases.models import Case
from oldp.apps.processing.admin_views import CreationDashboardView


class CaseCreationDashboardView(CreationDashboardView):
    """Dashboard showing case creation statistics with filtering and batch processing."""

    model = Case
    admin_changelist_url_name = "admin:cases_case_changelist"
    app_label = "cases"
    entity_name = "Case"
    entity_name_plural = "Cases"



import logging
from typing import List, Tuple

logger = logging.getLogger(__name__)


class BaseMarker(object):
    def get_start_position(self) -> int:
        raise NotImplementedError()

    def get_end_position(self) -> int:
        raise NotImplementedError()

    def get_marker_open_format(self) -> str:
        """Format of opening tag, e.g. [ref={uuid}]. Available placeholders: all marker class attributes.

        :return: format string
        """
        raise NotImplementedError()

    def get_marker_close_format(self) -> str:
        """Format of opening tag, e.g. [/ref]. Available placeholders: all marker class attributes.

        :return: format string
        """
        raise NotImplementedError()

    def get_marker_open(self):
        return self.get_marker_open_format().format(**self.__dict__)

    def get_marker_close(self):
        return self.get_marker_close_format().format(**self.__dict__)

    def insert_marker(self, content, marker_offset) -> Tuple[str, int]:
        """Replace the original content with markers, e.g. [ref]xy[/ref].

        :param content: Original content
        :param marker_offset: Offset from previous markers
        :return: Content with markers
        """
        start = self.get_start_position() + marker_offset
        end = self.get_end_position() + marker_offset

        # marker_open = '[ref=%i]' % key
        # Instead of key use uuid
        marker_open = self.get_marker_open()
        marker_close = self.get_marker_close()

        marker_offset += len(marker_open) + len(marker_close)

        # double replacements
        # alternative: content[start:end]
        content = (
            content[:start]
            + marker_open
            + content[start:end]
            + marker_close
            + content[end:]
        )

        return content, marker_offset


def insert_markers(content: str, markers: List[BaseMarker]):
    """Insert markers into content

    :param content: Without markers
    :param markers:
    :return:
    """
    marker_offset = 0
    content_with_markers = content
    sorted_markers = sorted(
        markers, key=lambda k: k.get_start_position()
    )  # order by occurrence in text

    for i, marker in enumerate(sorted_markers):
        # Check on overlaps
        if (
            i > 0
            and sorted_markers[i - 1].get_end_position() >= marker.get_start_position()
        ):
            logger.error("Marker overlaps with previous marker: %s" % marker)
        elif (
            i + 1 < len(sorted_markers)
            and sorted_markers[i + 1].get_start_position() <= marker.get_end_position()
        ):
            logger.error("Marker overlaps with next marker: %s" % marker)
        else:
            # Everything fine, replace content
            content_with_markers, marker_offset = marker.insert_marker(
                content_with_markers, marker_offset
            )

    return content_with_markers



# Generated manually for case creation API

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("accounts", "0002_apitokenpermission_apitokenpermissiongroup_and_more"),
        ("cases", "0021_remove_old_source_fields"),
    ]

    operations = [
        migrations.AddField(
            model_name="case",
            name="created_by_token",
            field=models.ForeignKey(
                blank=True,
                help_text="API token used to create this case via the API",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="created_cases",
                to="accounts.apitoken",
            ),
        ),
    ]



import logging

from oldp.apps.cases.models import Case
from oldp.apps.cases.processing.processing_steps import CaseProcessingStep
from oldp.apps.search.processing.processing_steps.generate_related import (
    BaseGenerateRelated,
)

logger = logging.getLogger(__name__)


class ProcessingStep(CaseProcessingStep, BaseGenerateRelated):
    description = "Set review_status=pending"

    def process(self, case: Case):
        case.review_status = "pending"

        return case



from django.contrib.auth.models import User
from django.test import Client, tag
from django.urls import reverse

from oldp.apps.cases.models import Case
from oldp.apps.lib.tests import ExtendedLiveServerTestCase


@tag("views")
class CasesViewsTestCase(ExtendedLiveServerTestCase):
    fixtures = [
        "locations/countries.json",
        "locations/states.json",
        "locations/cities.json",
        "courts/courts.json",
        "cases/cases.json",
    ]
    username = "test"
    password = "test"

    def setUp(self):
        self.user = User.objects.create_user(
            self.username, "test@example.com", self.password
        )

        self.user_client = Client()
        self.user_client.force_login(self.user)

        self.staff_user = User.objects.create_user(
            "staff", "staff@example.com", "staff", is_staff=True
        )

        self.staff_client = Client()
        self.staff_client.force_login(self.staff_user)

    def test_index(self):
        res = self.client.get(reverse("cases:index"))

        self.assertEqual(res.status_code, 200)

        self.assertContains(res, "another-awesome-case")
        self.assertContains(res, "foo-case")

        self.assertStringOrder(res, "foo-case", "another-awesome-case")

    def test_index_filter(self):
        res = self.client.get(reverse("cases:index") + "?court__state=1")

        self.assertNotContains(res, "another-awesome-case")
        self.assertContains(res, "foo-case")

    def test_detail(self):
        item = Case.objects.get(pk=1)

        res = self.client.get(item.get_absolute_url())

        self.assertContains(res, item.get_content_as_html(), count=1, status_code=200)

    def test_private_detail(self):
        private_item = Case.objects.get(pk=2)
        res = self.client.get(private_item.get_absolute_url())

        self.assertEqual(404, res.status_code, "Private content should not be found")

        res = self.staff_client.get(private_item.get_absolute_url())
        self.assertEqual(
            404,
            res.status_code,
            "Private content should return not found also for staff users",
        )

    def test_detail_as_user(self):
        item = Case.objects.get(pk=1)

        res = self.client.get(item.get_absolute_url())
        res_staff = self.staff_client.get(item.get_absolute_url())

        anon_content = item.get_content_as_html(request=res.wsgi_request)
        user_content = item.get_content_as_html(request=res_staff.wsgi_request)

        self.assertContains(res, anon_content, count=1, status_code=200)
        self.assertEqual(
            anon_content,
            user_content,
            "User and anon content should match (no user content available)",
        )

        # TODO Check on user annotations
        # self.assertContains(res, user_content, count=0, status_code=200)

    def test_short_url(self):
        item = Case.objects.get(pk=1)

        res = self.client.get(item.get_short_url())

        self.assertRedirects(res, item.get_absolute_url(), status_code=301)



import datetime
import html
import json
import logging

from django.conf import settings
from django.core import serializers
from django.core.serializers.base import DeserializationError
from django.db import models
from django.urls import reverse
from django.utils.html import strip_tags
from django.utils.text import slugify

# from django.utils.translation import gettext_lazy as _
from django.utils.translation import gettext_lazy as _

from oldp.apps.annotations.content_models import AnnotationContent
from oldp.apps.courts.models import Court
from oldp.apps.lib.markers import insert_markers
from oldp.apps.processing.errors import ProcessingError
from oldp.apps.references.content_models import ReferenceContent
from oldp.apps.search.models import RelatedContent, SearchableContent
from oldp.apps.sources.models import SourceContent

logger = logging.getLogger(__name__)


class Case(
    SourceContent, models.Model, SearchableContent, ReferenceContent, AnnotationContent
):
    """Model representing court cases (i.e. opinions, decisions, verdicts, ...)"""

    title = models.CharField(
        max_length=255,
        default="",
        blank=True,
        help_text="Title (currently not used due to copyright issues)",
    )
    slug = models.SlugField(
        max_length=200,
        unique=True,
        db_index=True,
        help_text="Used to urls (consists of court, date, file number)",
    )
    court = models.ForeignKey(
        Court,
        default=Court.DEFAULT_ID,
        help_text="Responsible court entity",
        on_delete=models.SET_DEFAULT,
    )
    court_raw = models.CharField(
        max_length=255,
        default="{}",
        help_text="Raw court information from crawler (JSON)",
    )
    court_chamber = models.CharField(
        max_length=150, null=True, blank=True, help_text="Court chamber (e.g. 1. Senat)"
    )
    date = models.DateField(
        null=True, db_index=True, help_text="Publication date as in source"
    )
    created_date = models.DateTimeField(
        auto_now_add=True,
        help_text="Entry is created at this date time",
        db_index=True,
    )
    updated_date = models.DateTimeField(
        auto_now=True,
        help_text="Date time of last change",
        db_index=True,
    )
    file_number = models.CharField(
        max_length=100,
        null=True,
        blank=True,
        help_text="File number as defined by court",
    )
    type = models.CharField(
        max_length=100,
        null=True,
        blank=True,
        help_text="Type of decision (Urteil, Beschluss, ...)",
        db_index=True,
    )
    pdf_url = models.URLField(
        # TODO Maybe we should store PDF files locally as well
        null=True,
        blank=True,
        max_length=255,
        help_text="URL to original PDF file (not in use)",
    )
    review_status = models.CharField(
        max_length=10,
        choices=[
            ("pending", "Pending"),
            ("accepted", "Accepted"),
            ("rejected", "Rejected"),
        ],
        default="accepted",
        db_index=True,
        help_text="Review status for case visibility",
    )
    raw = models.TextField(
        null=True,
        blank=True,
        help_text="Raw content (HTML) from crawler that can used to reconstruct all case information",
        editable=False,
    )
    abstract = models.TextField(
        null=True, blank=True, help_text="Case abstract (Leitsatz) formatted in HTML"
    )
    content = models.TextField(help_text="Case full-text formatted in Legal HTML")
    ecli = models.CharField(
        max_length=255,
        blank=True,
        verbose_name="ECLI",
        help_text="European Case Law Identifier",
    )
    preceding_cases = models.ManyToManyField(
        "self",
        blank=True,
        help_text="Cases from inferior courts (lower judicial authority)",
    )
    preceding_cases_raw = models.TextField(
        blank=True,
        null=True,
        help_text="Cases from inferior courts as in source HTML",
    )
    following_cases = models.ManyToManyField(
        "self",
        blank=True,
        help_text="Cases from superior courts (higher judicial authority)",
    )
    following_cases_raw = models.TextField(
        blank=True,
        null=True,
        help_text="Cases from inferior courts as in source HTML",
    )
    created_by_token = models.ForeignKey(
        "accounts.APIToken",
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="created_cases",
        help_text="API token used to create this case via the API",
    )

    # The following fields are excluded from the SELECT-query when querying the database
    defer_fields_list_view = [
        "court_raw",
        "pdf_url",
        "source_url",
        "source_file",
        "raw",
        "content",
        "preceding_cases",
        "preceding_cases_raw",
        "following_cases",
        "following_cases_raw",
        "court__description",
        "court__homepage",
        "court__image",
        "court__street_address",
        "court__postal_code",
        "court__address_locality",
        "court__telephone",
        "court__fax_number",
        "court__email",
    ]

    class Meta:
        ordering = ("-date",)
        unique_together = (("court", "file_number"),)
        indexes = [
            models.Index(fields=["ecli"], name="cases_case_ecli_idx"),
            models.Index(fields=["file_number"], name="cases_case_file_number_idx"),
            models.Index(fields=["court", "-date"], name="cases_case_court_date_idx"),
        ]
        # TODO court, year, file_number should be better

    def is_private(self):
        """Whether this item is not publicly visible (pending or rejected)."""
        return self.review_status != "accepted"

    def get_filename(self, ext="json"):
        return "%s.%s" % (self.slug, ext)

    def get_topics(self):
        # TODO
        return _("Unknown topic")

    def get_court_raw(self):
        """Court information from source"""
        return json.loads(self.court_raw)

    def get_type(self):
        return self.__class__.__name__

    def get_id(self):
        return self.id

    def get_content_as_html(self, request=None) -> str:
        """Content is stored in HTML (no conversion needed)

        :return: str
        """
        # TODO make line numbers clickable

        content = self.content

        if content is None or len(content) < 1:
            logger.warning("Content is not set or empty")

        markers = []

        # TODO db should return markers already in order
        markers += list(self.get_reference_markers())

        # Generic markers
        markers += list(self.get_markers(request))

        content = insert_markers(content, markers)

        return content

    def get_text(self) -> str:
        """Case content as plain text

        :return: plain-text
        """
        return strip_tags(html.unescape(self.content))

    def get_title(self) -> str:
        try:
            court_name = self.court.name

            # Attach chamber if available
            if self.court_chamber is not None and self.court_chamber != "":
                court_name += " (" + self.court_chamber + ")"
        except Court.DoesNotExist:
            court_name = "(no court)"

        return "%s vom %s - %s" % (self.get_case_type(), court_name, self.file_number)

    def get_short_title(self, max_length=75) -> str:
        title = self.get_title()
        if len(title) > max_length:
            return title[:max_length] + "..."
        else:
            return title

    def get_case_type(self):
        return self.type

    def get_date(self, date_format="%Y-%m-%d"):
        return self.date.strftime(date_format)

    def get_related(self, n=5):
        """Related items that are pre-computed with "generate_related_cases" command.

        :param n: number of items
        :return:
        """
        items = []
        for item in (
            RelatedCase.objects.filter(seed_content=self)
            .select_related("related_content")
            .order_by("-score")[:n]
        ):
            items.append(item.related_content)
        return items

    def get_short_url(self):
        return settings.SITE_URL + reverse("cases_short_url", args=(self.pk,))

    def get_absolute_url(self):
        if self.slug is None or self.slug == "":  # TODO added view by id
            self.slug = "no-slug"

        return reverse("cases:case", args=(self.slug,))

    def get_api_url(self):
        return "/api/cases/{}/".format(self.pk)

    def get_admin_url(self):
        return reverse("admin:cases_case_change", args=(self.pk,))

    def get_es_url(self):
        return (
            settings.ELASTICSEARCH_URL
            + settings.ELASTICSEARCH_INDEX
            + "/modelresult/cases.case.%s" % self.pk
        )

    def get_reference_marker_model(self):
        from oldp.apps.references.models import CaseReferenceMarker

        return CaseReferenceMarker

    def set_slug(self):
        # Transform date to string
        if isinstance(self.date, datetime.date):
            date_str = self.date.strftime("%Y-%m-%d")
        else:
            date_str = "%s" % self.date

        # File numbers can be lists, so limit the length
        max_fn_length = 20

        self.slug = (
            self.court.slug
            + "-"
            + date_str
            + "-"
            + slugify(self.file_number[:max_fn_length])
        )

    def set_ecli(self):
        """Generate ECLI from court code and file number

        See ECLI definition:

        Consists of:
        - ‘ECLI’: to identify the identifier as being a European Case Law Identifier;
        - the country code;
        - the code of the court that rendered the judgment;
        - the year the judgment was rendered;
        - an ordinal number, up to 25 alphanumeric characters, in a format that is decided upon by each Member State.
            Dots are allowed, but not other punctuation marks.

        """
        self.ecli = (
            "ECLI:de:"
            + self.court.code
            + ":"
            + str(self.date.year)
            + ":"
            + slugify(self.file_number)
        )

    def get_annotation_model(self):
        from oldp.apps.annotations.models import CaseAnnotation

        return CaseAnnotation

    def get_marker_model(self):
        from oldp.apps.annotations.models import CaseMarker

        return CaseMarker

    def __str__(self):
        return "<Case(#%i, court=%s, file_number=%s)>" % (
            self.pk,
            self.court.code,
            self.file_number,
        )

    def to_json(self, file_path=None) -> str:
        json_str = serializers.serialize("json", [self])

        if file_path is not None:
            with open(file_path, "w") as f:
                f.write(json_str)

        return json_str

    @staticmethod
    def from_json_file(file_path):
        with open(file_path) as f:
            out = serializers.deserialize("json", f.read())  # , ignorenonexistent=True)
            # print(len(out))

            try:
                for o in out:
                    return o.object
            except DeserializationError:
                pass

            raise ProcessingError("Cannot deserialize: %s" % file_path)

        # MySQL utf8mb4 bugfix
        # if instance.raw is not None:
        #     instance.raw = ''.join([char if ord(char) < 128 else '' for char in instance.raw])
        #
        # if instance.text is not None:
        #     instance.text = ''.join([char if ord(char) < 128 else '' for char in instance.text])
        #
        # return instance

    @staticmethod
    def get_queryset(request=None):
        # TODO superuser?
        if settings.DEBUG:
            return Case.objects.all()
        else:
            # production
            # hide non-accepted content
            return Case.objects.filter(review_status="accepted")


class RelatedCase(RelatedContent):
    seed_content = models.ForeignKey(
        Case, related_name="seed_id", on_delete=models.CASCADE
    )
    related_content = models.ForeignKey(
        Case, related_name="related_id", on_delete=models.CASCADE
    )

    class Meta:
        indexes = [
            models.Index(
                fields=["seed_content", "-score"], name="cases_relcase_seed_score_idx"
            ),
        ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2017-12-19 12:22
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("laws", "0001_initial"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="law",
            name="foo",
        ),
    ]



# Generated by Django 2.1.2 on 2018-12-03 10:57

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("courts", "0013_auto_20181128_1054"),
    ]

    operations = [
        migrations.RenameField(
            model_name="court",
            old_name="updated",
            new_name="updated_date",
        ),
        migrations.AddField(
            model_name="court",
            name="aliases",
            field=models.TextField(
                blank=True, help_text="List of aliases (one per line)", null=True
            ),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2018-02-09 16:04
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("courts", "0008_auto_20180209_1002"),
    ]

    operations = [
        migrations.AlterField(
            model_name="court",
            name="name",
            field=models.CharField(
                help_text="Full name of the court with location", max_length=200
            ),
        ),
        migrations.AlterField(
            model_name="court",
            name="wikipedia_title",
            field=models.CharField(
                blank=True,
                help_text="Title of the corresponding Wikipedia article",
                max_length=200,
                null=True,
            ),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2017-12-22 18:02
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("laws", "0002_remove_law_foo"),
    ]

    operations = [
        migrations.CreateModel(
            name="RelatedLaw",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("score", models.DecimalField(decimal_places=8, max_digits=12)),
                (
                    "related_content",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="related_id",
                        to="laws.Law",
                    ),
                ),
                (
                    "seed_content",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="seed_id",
                        to="laws.Law",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
    ]



from django.db.models import Q
from django_filters.rest_framework import DjangoFilterBackend
from rest_framework import viewsets
from rest_framework.exceptions import ValidationError
from rest_framework.filters import OrderingFilter

from oldp.api.permissions import OwnerPrivatePermission
from oldp.apps.annotations.filters import AnnotationLabelFilter, CaseAnnotationFilter
from oldp.apps.annotations.models import AnnotationLabel, CaseAnnotation, CaseMarker
from oldp.apps.annotations.serializers import (
    AnnotationLabelSerializer,
    CaseAnnotationSerializer,
    CaseMarkerSerializer,
)


class AnnotationLabelViewSet(viewsets.ModelViewSet):
    queryset = AnnotationLabel.objects.all()
    serializer_class = AnnotationLabelSerializer
    permission_classes = (OwnerPrivatePermission,)

    filter_backends = (
        OrderingFilter,
        DjangoFilterBackend,
    )
    filterset_class = AnnotationLabelFilter
    ordering_fields = (
        "created_at",
        "updated_at",
    )

    def get_queryset(self):
        qs = AnnotationLabel.objects.select_related("owner").order_by("owner")

        # public items or user is owner
        if hasattr(self, "request") and self.request.user.is_authenticated:
            if self.request.user.is_staff:
                return qs.all()
            else:
                return qs.filter(Q(private=False) | Q(owner=self.request.user))
        else:
            return qs.filter(private=False)

    def perform_create(self, serializer):
        if AnnotationLabel.objects.filter(
            owner=self.request.user, slug=serializer.validated_data.get("slug")
        ).exists():
            raise ValidationError(
                {
                    "slug": "The fields owner, slug must make a unique set. (slug: `%s`, owner: %s)"
                    % (
                        serializer.validated_data.get("slug"),
                        self.request.user.username,
                    )
                }
            )
        else:
            serializer.save(owner=self.request.user)


class CaseAnnotationViewSet(viewsets.ModelViewSet):
    queryset = CaseAnnotation.objects.select_related(
        "belongs_to__court", "label", "label__owner"
    ).order_by("label")
    serializer_class = CaseAnnotationSerializer
    permission_classes = (OwnerPrivatePermission,)

    filter_backends = (DjangoFilterBackend,)
    filterset_class = CaseAnnotationFilter

    def filter_queryset_by_permission(self, queryset):
        # public items or user is owner
        if hasattr(self, "request") and self.request.user.is_authenticated:
            if self.request.user.is_staff:
                return queryset.all()
            else:
                return queryset.filter(
                    Q(label__private=False) | Q(label__owner=self.request.user)
                )
        else:
            return queryset.filter(label__private=False)

    def get_queryset(self):
        return self.filter_queryset_by_permission(self.queryset)


class CaseMarkerViewSet(viewsets.ModelViewSet):
    queryset = CaseMarker.objects.select_related("belongs_to__court", "label").order_by(
        "label"
    )
    serializer_class = CaseMarkerSerializer
    permission_classes = (OwnerPrivatePermission,)

    filter_backends = (DjangoFilterBackend,)
    filterset_class = CaseAnnotationFilter



from django.conf import settings
from django.conf.urls.static import static
from django.contrib import admin
from django.contrib.flatpages import views as flatpages_views
from django.contrib.sitemaps import views as sitemaps_views
from django.urls import include, path, re_path
from django.views.decorators.cache import cache_page
from django.views.generic import RedirectView, TemplateView

from oldp.apps import cases
from oldp.apps.cases.admin_views import CaseCreationDashboardView
from oldp.apps.cases.sitemaps import CaseSitemap
from oldp.apps.courts.admin_views import CourtCreationDashboardView
from oldp.apps.courts.sitemaps import CourtSitemap
from oldp.apps.laws.admin_views import (
    LawBookCreationDashboardView,
    LawCreationDashboardView,
)
from oldp.apps.laws.sitemaps import LawSitemap
from oldp.apps.search.views import CustomSearchView, autocomplete_view
from oldp.utils.cache_per_user import cache_per_role

# Error handlers
handler500 = "oldp.apps.homepage.views.error500_view"
handler404 = "oldp.apps.homepage.views.error404_view"
handler403 = "oldp.apps.homepage.views.error_permission_denied_view"
handler400 = "oldp.apps.homepage.views.error_bad_request_view"


urlpatterns = [
    # Admin custom views (must be before admin.site.urls)
    path(
        "admin/cases/dashboard/",
        CaseCreationDashboardView.as_view(),
        name="cases_creation_dashboard",
    ),
    path(
        "admin/laws/dashboard/",
        LawCreationDashboardView.as_view(),
        name="laws_creation_dashboard",
    ),
    path(
        "admin/lawbooks/dashboard/",
        LawBookCreationDashboardView.as_view(),
        name="lawbooks_creation_dashboard",
    ),
    path(
        "admin/courts/dashboard/",
        CourtCreationDashboardView.as_view(),
        name="courts_creation_dashboard",
    ),
    # Admin
    re_path(r"^admin/", admin.site.urls),
    # Apps
    re_path(r"^case/", include("oldp.apps.cases.urls")),
    re_path(r"^c/(?P<pk>[0-9]+)$", cases.views.short_url_view, name="cases_short_url"),
    re_path(r"^law/", include("oldp.apps.laws.urls")),
    re_path(r"^court/", include("oldp.apps.courts.urls")),
    re_path(r"^accounts/", include("oldp.apps.accounts.urls")),
    re_path(r"^accounts/", include("allauth.urls")),
    re_path(r"^contact/", include("oldp.apps.contact.urls")),
    re_path(r"^search/autocomplete", autocomplete_view),
    re_path(
        r"^search/",
        cache_per_role(settings.CACHE_TTL)(CustomSearchView.as_view()),
        name="haystack_search",
    ),
    re_path(r"^sources/", include("oldp.apps.sources.urls")),
    # Files
    re_path(
        r"^favicon\.ico$",
        RedirectView.as_view(url="/static/favicon.ico", permanent=True),
    ),
    re_path(
        r"^robots\.txt$",
        TemplateView.as_view(template_name="robots.txt", content_type="text/plain"),
    ),
    # Third-party apps
    # re_path(r'^tellme/', include("tellme.urls")),
    # API
    re_path(r"^api/", include("oldp.api.urls")),
    # Homepage
    re_path(r"^", include("oldp.apps.homepage.urls")),
    re_path(r"^pages(?P<url>.*/)$", flatpages_views.flatpage, name="flatpages"),
] + static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)

# Sitemaps
sitemaps = {
    "court": CourtSitemap(),
    "case": CaseSitemap(),
    "law": LawSitemap(),
}

urlpatterns += [
    path(
        "sitemap.xml", cache_page(86400)(sitemaps_views.index), {"sitemaps": sitemaps}
    ),
    path(
        "sitemap-<section>.xml",
        cache_page(86400)(sitemaps_views.sitemap),
        {"sitemaps": sitemaps},
        name="django.contrib.sitemaps.views.sitemap",
    ),
]


# DEBUG only views
if settings.DEBUG and "debug_toolbar" in settings.INSTALLED_APPS:
    # Django debug toolbar
    import debug_toolbar

    urlpatterns = [
        re_path(r"^__debug__/", include(debug_toolbar.urls)),
    ] + urlpatterns

# Django Silk profiling (toggled via PROFILING_ENABLED env var)
if "silk" in settings.INSTALLED_APPS:
    urlpatterns = [
        path("silk/", include("silk.urls", namespace="silk")),
    ] + urlpatterns

# """
# URL configuration for oldp project.

# The `urlpatterns` list routes URLs to views. For more information please see:
#     https://docs.djangoproject.com/en/5.0/topics/http/urls/
# Examples:
# Function views
#     1. Add an import:  from my_app import views
#     2. Add a URL to urlpatterns:  path('', views.home, name='home')
# Class-based views
#     1. Add an import:  from other_app.views import Home
#     2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
# Including another URLconf
#     1. Import the include() function: from django.urls import include, path
#     2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
# """
# from django.contrib import admin
# from django.urls import path

# urlpatterns = [
#     path('admin/', admin.site.urls),
# ]



# Generated by Django 2.1.2 on 2019-01-28 10:51

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("cases", "0017_case_source_file"),
    ]

    operations = [
        migrations.AlterField(
            model_name="case",
            name="created_date",
            field=models.DateTimeField(
                auto_now_add=True,
                db_index=True,
                help_text="Entry is created at this date time",
            ),
        ),
        migrations.AlterField(
            model_name="case",
            name="source_name",
            field=models.CharField(
                db_index=True,
                help_text="Name of source (crawler class)",
                max_length=100,
            ),
        ),
        migrations.AlterField(
            model_name="case",
            name="type",
            field=models.CharField(
                blank=True,
                db_index=True,
                help_text="Type of decision (Urteil, Beschluss, ...)",
                max_length=100,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="case",
            name="updated_date",
            field=models.DateTimeField(
                auto_now=True, db_index=True, help_text="Date time of last change"
            ),
        ),
    ]



"""Utility helpers for the search app."""


def is_search_backend_error(exc: Exception) -> bool:
    """Check if an exception is an Elasticsearch connection/transport error."""
    try:
        from elasticsearch.exceptions import ConnectionError, TransportError

        return isinstance(exc, (ConnectionError, TransportError))
    except ImportError:
        return False



# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2018-02-09 09:26
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("cases", "0002_auto_20171221_1055"),
    ]

    operations = [
        migrations.AlterField(
            model_name="case",
            name="court_chamber",
            field=models.CharField(blank=True, max_length=150, null=True),
        ),
        migrations.AlterField(
            model_name="case",
            name="court_raw",
            field=models.CharField(default="{}", max_length=255),
        ),
        migrations.AlterField(
            model_name="case",
            name="ecli",
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AlterField(
            model_name="case",
            name="pdf_url",
            field=models.URLField(blank=True, max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name="case",
            name="raw",
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name="case",
            name="source_url",
            field=models.URLField(max_length=255),
        ),
        migrations.AlterField(
            model_name="case",
            name="title",
            field=models.CharField(blank=True, default="", max_length=255),
        ),
    ]



# Generated by Django 5.1.4 on 2026-01-16 12:22

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("accounts", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="APITokenPermission",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "resource",
                    models.CharField(
                        choices=[
                            ("cases", "Cases"),
                            ("laws", "Laws"),
                            ("courts", "Courts"),
                            ("lawbooks", "Law Books"),
                            ("references", "References"),
                            ("annotations", "Annotations"),
                        ],
                        help_text="The resource this permission applies to",
                        max_length=50,
                        verbose_name="Resource",
                    ),
                ),
                (
                    "action",
                    models.CharField(
                        choices=[
                            ("read", "Read"),
                            ("write", "Write"),
                            ("delete", "Delete"),
                        ],
                        help_text="The action allowed on this resource",
                        max_length=20,
                        verbose_name="Action",
                    ),
                ),
                (
                    "description",
                    models.TextField(
                        blank=True,
                        help_text="Optional description of what this permission allows",
                        verbose_name="Description",
                    ),
                ),
            ],
            options={
                "verbose_name": "API Token Permission",
                "verbose_name_plural": "API Token Permissions",
                "ordering": ["resource", "action"],
            },
        ),
        migrations.CreateModel(
            name="APITokenPermissionGroup",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        help_text="Unique name for this permission group",
                        max_length=100,
                        unique=True,
                        verbose_name="Name",
                    ),
                ),
                (
                    "description",
                    models.TextField(
                        blank=True,
                        help_text="Description of what this permission group allows",
                        verbose_name="Description",
                    ),
                ),
                (
                    "is_default",
                    models.BooleanField(
                        default=False,
                        help_text="Whether this is the default permission group for new tokens",
                        verbose_name="Is Default",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(auto_now_add=True, verbose_name="Created"),
                ),
                (
                    "updated",
                    models.DateTimeField(auto_now=True, verbose_name="Updated"),
                ),
            ],
            options={
                "verbose_name": "API Token Permission Group",
                "verbose_name_plural": "API Token Permission Groups",
                "ordering": ["name"],
            },
        ),
        migrations.RenameIndex(
            model_name="apitoken",
            new_name="accounts_ap_user_id_42f198_idx",
            old_name="accounts_ap_user_id_ac6ac5_idx",
        ),
        migrations.RenameIndex(
            model_name="apitoken",
            new_name="accounts_ap_expires_46b4e5_idx",
            old_name="accounts_ap_expires_e66c1e_idx",
        ),
        migrations.AlterField(
            model_name="apitoken",
            name="scopes",
            field=models.JSONField(
                blank=True,
                default=list,
                help_text="Deprecated: Use permission_group instead. List of scopes this token has access to",
                verbose_name="Scopes",
            ),
        ),
        migrations.AlterUniqueTogether(
            name="apitokenpermission",
            unique_together={("resource", "action")},
        ),
        migrations.AddField(
            model_name="apitokenpermissiongroup",
            name="permissions",
            field=models.ManyToManyField(
                blank=True,
                help_text="Permissions included in this group",
                related_name="permission_groups",
                to="accounts.apitokenpermission",
                verbose_name="Permissions",
            ),
        ),
        migrations.AddField(
            model_name="apitoken",
            name="permission_group",
            field=models.ForeignKey(
                blank=True,
                help_text="The permission group assigned to this token (defines what resources it can access)",
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="tokens",
                to="accounts.apitokenpermissiongroup",
                verbose_name="Permission Group",
            ),
        ),
    ]



"""Tests for review_status filtering in search API."""

from unittest.mock import MagicMock, patch

from django.test import TestCase

from oldp.apps.cases.models import Case
from oldp.apps.search.api import SearchViewMixin


class SearchViewMixinReviewStatusTestCase(TestCase):
    """Tests that SearchViewMixin.get_queryset() filters by review_status."""

    def test_get_queryset_filters_accepted_only(self):
        """SearchViewMixin applies review_status=accepted filter to SearchQuerySet."""
        mixin = SearchViewMixin()
        mixin.search_models = [Case]

        with patch("oldp.apps.search.api.SearchQuerySet") as mock_sqs_cls:
            mock_sqs = MagicMock()
            mock_sqs_cls.return_value = mock_sqs
            mock_sqs.models.return_value = mock_sqs
            mock_sqs.filter.return_value = mock_sqs

            result = mixin.get_queryset()

            mock_sqs.models.assert_called_once_with(Case)
            mock_sqs.filter.assert_called_once_with(review_status="accepted")
            self.assertEqual(result, mock_sqs)

    def test_get_queryset_without_models_still_filters(self):
        """SearchViewMixin applies review_status filter even without search_models."""
        mixin = SearchViewMixin()
        mixin.search_models = []

        with patch("oldp.apps.search.api.SearchQuerySet") as mock_sqs_cls:
            mock_sqs = MagicMock()
            mock_sqs_cls.return_value = mock_sqs
            mock_sqs.filter.return_value = mock_sqs

            result = mixin.get_queryset()

            mock_sqs.models.assert_not_called()
            mock_sqs.filter.assert_called_once_with(review_status="accepted")
            self.assertEqual(result, mock_sqs)



from django.test import LiveServerTestCase, tag
from django.urls import reverse


@tag("views")
class LawsViewsTestCase(LiveServerTestCase):
    fixtures = ["laws/laws.json"]

    def test_index(self):
        res = self.client.get(reverse("laws:index"))

        self.assertContains(res, "Grundgesetz")
        self.assertContains(res, "aappro-2002")

    def test_index_char(self):
        res = self.client.get(reverse("laws:index_char", args=("g",)))

        self.assertContains(res, "Grundgesetz")

    def test_book(self):
        res = self.client.get(reverse("laws:book", args=("gg",)))

        self.assertContains(res, "Grundgesetz")

    def test_book_revision(self):
        res = self.client.get(
            reverse("laws:book", args=("gg",)) + "?revision_date=2010-07-26"
        )

        self.assertContains(res, "Grundgesetz")

    def test_law(self):
        res = self.client.get(reverse("laws:law", args=("gg", "artikel-1")))

        self.assertContains(
            res,
            "Die nachfolgenden Grundrechte binden Gesetzgebung, vollziehende Gewalt und "
            "Rechtsprechung als unmittelbar geltendes Recht",
        )



"""Custom admin views for court statistics dashboard."""

from oldp.apps.courts.models import Court
from oldp.apps.processing.admin_views import CreationDashboardView


class CourtCreationDashboardView(CreationDashboardView):
    """Dashboard showing court creation statistics with filtering and batch processing."""

    model = Court
    admin_changelist_url_name = "admin:courts_court_changelist"
    app_label = "courts"
    entity_name = "Court"
    entity_name_plural = "Courts"



from django.apps import AppConfig
from django.conf import settings
from django.utils import translation


class LibConfig(AppConfig):
    name = "oldp.apps.lib"

    def ready(self):
        from oldp.apps.lib.templatetags import qstring, string_filters

        if qstring or string_filters:
            pass


DEBUG_CONTENT = None


class DomainLocaleMiddleware(object):
    """Change language based on domain.

    Taken from: https://7webpages.com/blog/switch-language-regarding-of-domain-in-django/

    """

    def __init__(self, get_response):
        self.get_response = get_response
        # One-time configuration and initialization.

    def __call__(self, request):
        # Code to be executed for each request before
        # the view (and later middleware) are called.
        if "HTTP_ACCEPT_LANGUAGE" in request.META:
            # Totally ignore the browser settings...
            del request.META["HTTP_ACCEPT_LANGUAGE"]

        if "HTTP_HOST" in request.META:
            current_domain = request.META["HTTP_HOST"]
            lang_code = settings.LANGUAGES_DOMAINS.get(current_domain)

            if lang_code:
                translation.activate(lang_code)
                request.LANGUAGE_CODE = lang_code

        response = self.get_response(request)

        # Code to be executed for each request/response after
        # the view is called.

        return response


class Counter:
    """Helper class for counter in templates

    {{ counter.increment }}
    {{ counter.count }}

    """

    count = 0

    def __init__(self, count=0):
        self.count = count

    def increment(self):
        self.count += 1
        return ""

    def decrement(self):
        self.count -= 1
        return ""


def unset_debug_content():
    global DEBUG_CONTENT

    DEBUG_CONTENT = None


def set_debug_content(public_message, dev_message):
    global DEBUG_CONTENT

    DEBUG_CONTENT = {"public_message": public_message, "dev_message": dev_message}



# Generated by Django 2.1.2 on 2018-11-05 14:42

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("cases", "0012_auto_20181025_1014"),
    ]

    operations = [
        migrations.AddField(
            model_name="case",
            name="following_cases",
            field=models.ManyToManyField(
                blank=True,
                help_text="Cases from superior courts (higher judicial authority)",
                related_name="_case_following_cases_+",
                to="cases.Case",
            ),
        ),
        migrations.AddField(
            model_name="case",
            name="following_cases_raw",
            field=models.TextField(
                blank=True,
                help_text="Cases from inferior courts as in source HTML",
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="case",
            name="preceding_cases",
            field=models.ManyToManyField(
                blank=True,
                help_text="Cases from inferior courts (lower judicial authority)",
                related_name="_case_preceding_cases_+",
                to="cases.Case",
            ),
        ),
        migrations.AddField(
            model_name="case",
            name="preceding_cases_raw",
            field=models.TextField(
                blank=True,
                help_text="Cases from inferior courts as in source HTML",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="case",
            name="court_raw",
            field=models.CharField(
                default="{}",
                help_text="Raw court information from crawler (JSON)",
                max_length=255,
            ),
        ),
    ]



import logging
import os
import re

from django.core.management import BaseCommand
from django.http import HttpRequest

from oldp.apps.homepage.views import error404_view, error500_view, settings

logger = logging.getLogger(__name__)

render_dir = os.path.join(settings.ASSETS_DIR, "html_pages")

views_to_render = {
    "error404.html": error404_view,
    "error500.html": error500_view,
}


class Command(BaseCommand):
    """pre-render html pages (for non-django content/display directly with web server/nginx)

    landing page
    error pages
        - use homepage error views

    """

    help = "Render static html pages"

    def __init__(self):
        super(Command, self).__init__()

    def handle(self, *args, **options):
        if not os.path.isdir(render_dir):
            os.mkdir(render_dir)
            logger.info("Output directory created: %s" % render_dir)

        # Compile SASS
        css_str = ""

        # Init request
        # TODO handle locale
        request = HttpRequest()

        for file_name in views_to_render:
            view_func = views_to_render[file_name]
            view_content = view_func(request).content.decode("utf-8")

            # Write view content to file
            with open(os.path.join(render_dir, file_name), "w") as f:
                logger.debug("Writing view to: %s" % file_name)

                # Replace CSS link with string (re.sub does not work with complex replace string)
                match = re.search(r'<link rel="stylesheet" href="(.*?)">', view_content)
                if match:
                    view_content = (
                        view_content[: match.start(0)]
                        + "<style>"
                        + css_str
                        + "</style>"
                        + view_content[match.end(0) :]
                    )

                f.write(view_content)

        logger.info("done")



from django.contrib.sitemaps import GenericSitemap

from oldp.apps.laws.models import Law


class LawSitemap(GenericSitemap):
    def __init__(self):
        super().__init__(
            {
                "queryset": Law.objects.select_related("book")
                .filter(book__latest=True)
                .defer("content", "footnotes")
                .order_by("-updated_date"),
                "date_field": "updated_date",
            },
            priority=0.9,
        )



# Generated by Django 2.1.1 on 2018-09-18 08:47

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("cases", "0009_auto_20180430_1225"),
    ]

    operations = [
        migrations.AddField(
            model_name="case",
            name="abstract",
            field=models.TextField(
                blank=True,
                help_text="Case abstract (Leitsatz) formatted in Legal Markdown",
                null=True,
            ),
        ),
    ]



from django.contrib.auth.models import User
from django.test import TestCase
from rest_framework.authtoken.models import Token
from rest_framework.test import APIClient

from oldp.apps.accounts.models import APIToken


class MeApiTestCase(TestCase):
    """Tests for the /api/me/ endpoint."""

    def setUp(self):
        self.user = User.objects.create_user(
            "testuser", "test@example.com", "testpassword", is_staff=True
        )
        self.client = APIClient()

    def test_unauthenticated_returns_401(self):
        res = self.client.get("/api/me/")
        self.assertEqual(res.status_code, 401)

    def test_session_auth_returns_user_info(self):
        self.client.login(username="testuser", password="testpassword")
        res = self.client.get("/api/me/")
        self.assertEqual(res.status_code, 200)
        data = res.json()
        self.assertEqual(data["id"], self.user.id)
        self.assertEqual(data["username"], "testuser")
        self.assertEqual(data["email"], "test@example.com")
        self.assertTrue(data["is_staff"])
        self.assertFalse(data["is_superuser"])
        self.assertEqual(data["auth_type"], "session")
        self.assertNotIn("token", data)

    def test_token_auth_returns_user_info(self):
        # Token is auto-created by signal on user creation
        token = Token.objects.get(user=self.user)
        self.client.credentials(HTTP_AUTHORIZATION=f"Token {token.key}")
        res = self.client.get("/api/me/")
        self.assertEqual(res.status_code, 200)
        data = res.json()
        self.assertEqual(data["username"], "testuser")
        self.assertEqual(data["auth_type"], "token")
        self.assertNotIn("token", data)

    def test_api_token_auth_returns_token_details(self):
        api_token = APIToken.objects.create(
            user=self.user,
            name="Test Token",
        )
        self.client.credentials(HTTP_AUTHORIZATION=f"Token {api_token.key}")
        res = self.client.get("/api/me/")
        self.assertEqual(res.status_code, 200)
        data = res.json()
        self.assertEqual(data["username"], "testuser")
        self.assertEqual(data["auth_type"], "api_token")
        self.assertIn("token", data)
        self.assertEqual(data["token"]["name"], "Test Token")
        self.assertIn("permissions", data["token"])
        self.assertIn("created", data["token"])

    def test_only_get_allowed(self):
        self.client.login(username="testuser", password="testpassword")
        for method in ["post", "put", "patch", "delete"]:
            res = getattr(self.client, method)("/api/me/")
            self.assertEqual(
                res.status_code, 405, f"{method.upper()} should not be allowed"
            )



from functools import wraps

from django.core.cache import cache
from django.template.response import TemplateResponse


def _response_has_vary_cookie(response):
    vary_header = response.get("Vary", "")
    if not vary_header:
        return False
    return "cookie" in {item.strip().lower() for item in vary_header.split(",")}


def cache_per_role(ttl=None, cache_post=False):
    """Cache view response per user role (anonymous, authenticated, staff).

    Unlike cache_per_user which creates one entry per user ID, this creates
    at most 3 entries per URL path — suitable for views with no per-individual-user content.
    """

    def decorator(function):
        @wraps(function)
        def apply_cache(request, *args, **kwargs):
            user = getattr(request, "user", None)
            if not user or not user.is_authenticated:
                role = "anon"
            elif user.is_staff:
                role = "staff"
            else:
                role = "auth"

            try:
                host = request.get_host()
            except Exception:
                host = request.META.get("HTTP_HOST", "unknown")
            lang = getattr(request, "LANGUAGE_CODE", None) or "default"
            method = "GET" if request.method == "HEAD" else request.method
            CACHE_KEY = "view_cache_%s_%s_%s_%s_%s" % (
                host,
                lang,
                method,
                request.get_full_path(),
                role,
            )

            if not cache_post and request.method == "POST":
                can_cache = False
            else:
                can_cache = True

            if can_cache:
                response = cache.get(CACHE_KEY, None)
            else:
                response = None

            if not response:
                response = function(request, *args, **kwargs)

                if isinstance(response, TemplateResponse):
                    response = response.render()

                has_set_cookie = response.has_header("Set-Cookie") or bool(
                    getattr(response, "cookies", None)
                )
                response_is_cacheable = (
                    response.status_code == 200
                    and not getattr(response, "streaming", False)
                    and not has_set_cookie
                    and not _response_has_vary_cookie(response)
                )
                if can_cache and response_is_cacheable:
                    cache.set(CACHE_KEY, response, ttl)
            return response

        return apply_cache

    return decorator


def cache_per_user(ttl=None, prefix=None, cache_post=False):
    """Based on https://djangosnippets.org/snippets/2524/

    Decorator for page caching based on user authentication with special cache for guests.

    :param ttl:
    :param prefix:
    :param cache_post: If POST-request should be cached as well
    :return:
    """

    def decorator(function):
        @wraps(function)
        def apply_cache(request, *args, **kwargs):
            # Define user
            if not request.user.is_authenticated:
                user = "anonymous"
            else:
                user = request.user.id

            # Set cache key
            if prefix:
                CACHE_KEY = "%s_%s" % (prefix, user)
            else:
                # Use url path + query parameters as cache key
                CACHE_KEY = "view_cache_%s_%s" % (request.get_full_path(), user)

            # Check on POST
            if not cache_post and request.method == "POST":
                can_cache = False
            else:
                can_cache = True

            if can_cache:
                response = cache.get(CACHE_KEY, None)
            else:
                response = None

            if not response:
                response = function(request, *args, **kwargs)

                # Render response when decorator is used on ListViews
                if isinstance(response, TemplateResponse):
                    response = response.render()

                if can_cache:
                    cache.set(CACHE_KEY, response, ttl)
            return response

        return apply_cache

    return decorator



"""Unit tests for utility functions in oldp.utils module.

Tests cover:
- find_from_mapping function
- get_elasticsearch_from_url function
- get_elasticsearch_settings_from_url function
"""

import logging

from django.test import TestCase, tag

from oldp.utils import (
    find_from_mapping,
    get_elasticsearch_from_url,
    get_elasticsearch_settings_from_url,
)

logger = logging.getLogger(__name__)


@tag("utils")
class FindFromMappingTestCase(TestCase):
    """Tests for the find_from_mapping function."""

    def test_find_exact_match(self):
        """Test finding an exact match in the mapping."""
        haystack = "Verwaltungsgericht Berlin"
        mapping = {"Verwaltungsgericht": "VG", "Oberlandesgericht": "OLG"}

        result = find_from_mapping(haystack, mapping)

        self.assertEqual(result, "VG")

    def test_find_case_insensitive(self):
        """Test case-insensitive matching."""
        haystack = "verwaltungsgericht Berlin"
        mapping = {"Verwaltungsgericht": "VG"}

        result = find_from_mapping(haystack, mapping)

        self.assertEqual(result, "VG")

    def test_find_with_word_boundaries(self):
        """Test that matching respects word boundaries."""
        haystack = "This is a test"
        mapping = {"test": "found", "tes": "partial"}

        result = find_from_mapping(haystack, mapping)

        # Should match "test" not "tes" due to word boundary
        self.assertEqual(result, "found")

    def test_find_no_match_returns_default(self):
        """Test that no match returns the default value."""
        haystack = "Something completely different"
        mapping = {"Verwaltungsgericht": "VG"}

        result = find_from_mapping(haystack, mapping, default="UNKNOWN")

        self.assertEqual(result, "UNKNOWN")

    def test_find_no_match_returns_none_by_default(self):
        """Test that no match returns None when no default is specified."""
        haystack = "Something completely different"
        mapping = {"Verwaltungsgericht": "VG"}

        result = find_from_mapping(haystack, mapping)

        self.assertIsNone(result)

    def test_find_mapping_list_true(self):
        """Test find_from_mapping with mapping_list=True returns the key."""
        haystack = "Verwaltungsgericht Berlin"
        mapping = {"Verwaltungsgericht": "VG", "Oberlandesgericht": "OLG"}

        result = find_from_mapping(haystack, mapping, mapping_list=True)

        # Should return the key, not the value
        self.assertEqual(result, "Verwaltungsgericht")

    def test_find_first_match_in_order(self):
        """Test that the first matching key is returned (dict order)."""
        haystack = "Oberlandesgericht Verwaltungsgericht"
        # In Python 3.7+, dict maintains insertion order
        mapping = {"Verwaltungsgericht": "VG", "Oberlandesgericht": "OLG"}

        result = find_from_mapping(haystack, mapping)

        # First key in mapping that matches should be returned
        self.assertEqual(result, "VG")

    def test_find_empty_mapping(self):
        """Test with empty mapping."""
        haystack = "Some text"
        mapping = {}

        result = find_from_mapping(haystack, mapping)

        self.assertIsNone(result)

    def test_find_empty_haystack(self):
        """Test with empty haystack."""
        haystack = ""
        mapping = {"test": "value"}

        result = find_from_mapping(haystack, mapping)

        self.assertIsNone(result)

    def test_find_with_special_regex_characters(self):
        """Test that special regex characters in keys are escaped."""
        # Note: Word boundary \b doesn't work with non-word chars like §
        # So we test with a word character key instead
        haystack = "Section 123 BGB applies here"
        mapping = {"Section 123": "section_123", "BGB": "civil_code"}

        result = find_from_mapping(haystack, mapping)

        self.assertEqual(result, "section_123")

    def test_find_word_boundary_prevents_partial_match(self):
        """Test that word boundaries prevent partial matches."""
        haystack = "Amtsgericht is here"
        mapping = {"Amt": "office"}

        result = find_from_mapping(haystack, mapping)

        # "Amt" should not match "Amtsgericht" due to word boundary
        self.assertIsNone(result)

    def test_find_with_umlauts(self):
        """Test matching with German umlauts."""
        haystack = "Münchener Gericht"
        mapping = {"Münchener": "MUC", "Berliner": "BER"}

        result = find_from_mapping(haystack, mapping)

        self.assertEqual(result, "MUC")

    def test_find_multiple_words_key(self):
        """Test matching with multi-word keys."""
        haystack = "Das Bundesverwaltungsgericht hat entschieden"
        mapping = {
            "Bundesverwaltungsgericht": "BVerwG",
            "Verwaltungsgericht": "VG",
        }

        result = find_from_mapping(haystack, mapping)

        self.assertEqual(result, "BVerwG")


@tag("utils")
class GetElasticsearchFromUrlTestCase(TestCase):
    """Tests for the get_elasticsearch_from_url function."""

    def test_parse_http_url(self):
        """Test parsing HTTP URL."""
        url = "http://localhost:9200/myindex"

        scheme, host, port, index = get_elasticsearch_from_url(url)

        self.assertEqual(scheme, "http")
        self.assertEqual(host, "localhost")
        self.assertEqual(port, 9200)
        self.assertEqual(index, "myindex")

    def test_parse_https_url(self):
        """Test parsing HTTPS URL."""
        url = "https://es.example.com:9243/production_index"

        scheme, host, port, index = get_elasticsearch_from_url(url)

        self.assertEqual(scheme, "https")
        self.assertEqual(host, "es.example.com")
        self.assertEqual(port, 9243)
        self.assertEqual(index, "production_index")

    def test_parse_url_with_subdomain(self):
        """Test parsing URL with subdomain."""
        url = "https://search.api.example.com:443/legal_index"

        scheme, host, port, index = get_elasticsearch_from_url(url)

        self.assertEqual(host, "search.api.example.com")
        self.assertEqual(port, 443)
        self.assertEqual(index, "legal_index")

    def test_parse_url_default_http_port(self):
        """Test parsing URL with default HTTP port."""
        url = "http://localhost:80/index"

        scheme, host, port, index = get_elasticsearch_from_url(url)

        self.assertEqual(port, 80)

    def test_parse_url_with_trailing_slash_returns_empty_index(self):
        """Test that URL with trailing slash returns empty index."""
        url = "http://localhost:9200/"

        scheme, host, port, index = get_elasticsearch_from_url(url)

        # Trailing slash results in empty string index (path splits to ['', ''])
        self.assertEqual(index, "")

    def test_parse_url_no_path_raises_error(self):
        """Test that URL without path raises ValueError."""
        url = "http://localhost:9200"

        with self.assertRaises(ValueError) as context:
            get_elasticsearch_from_url(url)

        self.assertIn("Cannot extract index", str(context.exception))

    def test_parse_url_with_ip_address(self):
        """Test parsing URL with IP address."""
        url = "http://192.168.1.100:9200/test_index"

        scheme, host, port, index = get_elasticsearch_from_url(url)

        self.assertEqual(host, "192.168.1.100")
        self.assertEqual(port, 9200)
        self.assertEqual(index, "test_index")


@tag("utils")
class GetElasticsearchSettingsFromUrlTestCase(TestCase):
    """Tests for the get_elasticsearch_settings_from_url function."""

    def test_settings_http_url(self):
        """Test generating settings from HTTP URL."""
        url = "http://localhost:9200/myindex"

        settings = get_elasticsearch_settings_from_url(url)

        self.assertEqual(settings["scheme"], "http")
        self.assertEqual(settings["host"], "localhost")
        self.assertEqual(settings["port"], 9200)
        self.assertEqual(settings["index"], "myindex")
        self.assertFalse(settings["use_ssl"])
        self.assertEqual(settings["urls"], [url])

    def test_settings_https_url_enables_ssl(self):
        """Test that HTTPS URL enables SSL."""
        url = "https://es.example.com:9243/secure_index"

        settings = get_elasticsearch_settings_from_url(url)

        self.assertEqual(settings["scheme"], "https")
        self.assertTrue(settings["use_ssl"])

    def test_settings_contains_all_required_keys(self):
        """Test that settings contains all required keys."""
        url = "http://localhost:9200/index"

        settings = get_elasticsearch_settings_from_url(url)

        required_keys = ["scheme", "host", "port", "index", "use_ssl", "urls"]
        for key in required_keys:
            self.assertIn(key, settings)

    def test_settings_urls_list_format(self):
        """Test that urls is a list containing the original URL."""
        url = "http://localhost:9200/index"

        settings = get_elasticsearch_settings_from_url(url)

        self.assertIsInstance(settings["urls"], list)
        self.assertEqual(len(settings["urls"]), 1)
        self.assertEqual(settings["urls"][0], url)



"""Unit tests for the Case Creation API.

Tests cover:
- Successful case creation
- Court resolution from name
- Duplicate detection (409 Conflict)
- Validation errors
- Reference extraction (optional)
- API token tracking
- Authentication and permissions
"""

from datetime import date
from unittest.mock import MagicMock, patch

from django.contrib.auth import get_user_model
from django.test import TestCase, override_settings
from rest_framework import status
from rest_framework.test import APIClient, APITestCase

from oldp.apps.accounts.models import (
    APIToken,
    APITokenPermission,
    APITokenPermissionGroup,
)
from oldp.apps.cases.exceptions import CourtNotFoundError, DuplicateCaseError
from oldp.apps.cases.models import Case
from oldp.apps.cases.serializers import CaseCreateSerializer
from oldp.apps.cases.services import CaseCreator, CourtResolver
from oldp.apps.courts.models import Court
from oldp.apps.sources.models import Source

User = get_user_model()


class CourtResolverTestCase(TestCase):
    """Tests for the CourtResolver service."""

    fixtures = [
        "locations/countries.json",
        "locations/states.json",
        "locations/cities.json",
        "courts/courts.json",
    ]

    def setUp(self):
        self.resolver = CourtResolver()

    def test_remove_chamber_with_number(self):
        """Test chamber extraction with numbered chamber."""
        name = "LG Koblenz 14. Zivilkammer"
        result_name, chamber = self.resolver.remove_chamber(name)
        self.assertEqual(result_name, "LG Koblenz")
        self.assertIsNotNone(chamber)
        self.assertIn("14", chamber)

    def test_remove_chamber_senat(self):
        """Test chamber extraction with Senat designation."""
        name = "OLG Koblenz 2. Senat für Bußgeldsachen"
        result_name, chamber = self.resolver.remove_chamber(name)
        self.assertEqual(result_name, "OLG Koblenz")
        self.assertIsNotNone(chamber)

    def test_remove_chamber_no_chamber(self):
        """Test chamber extraction when no chamber present."""
        name = "Amtsgericht Berlin"
        result_name, chamber = self.resolver.remove_chamber(name)
        self.assertEqual(result_name, "Amtsgericht Berlin")
        self.assertIsNone(chamber)

    def test_find_court_by_code(self):
        """Test finding court by code."""
        # Assuming there's a court with code in fixtures
        courts = Court.objects.exclude(code="").exclude(code__isnull=True)
        if courts.exists():
            court = courts.first()
            found = self.resolver.find_court("", court.code)
            self.assertEqual(found.pk, court.pk)

    def test_find_court_not_found_raises_error(self):
        """Test that non-existent court raises CourtNotFoundError."""
        with self.assertRaises(CourtNotFoundError):
            self.resolver.find_court("Nonexistent Court XYZ 12345")

    def test_find_court_empty_name_raises_error(self):
        """Test that empty court name raises CourtNotFoundError."""
        with self.assertRaises(CourtNotFoundError):
            self.resolver.find_court("")

    def test_resolve_extracts_chamber(self):
        """Test that resolve method extracts chamber and finds court."""
        # Get a valid court from fixtures
        court = Court.objects.exclude(pk=Court.DEFAULT_ID).first()
        if court:
            # Mock find_court to return the court
            with patch.object(self.resolver, "find_court", return_value=court):
                found_court, chamber = self.resolver.resolve("LG Test 14. Zivilkammer")
                self.assertEqual(found_court, court)


class CaseCreatorTestCase(TestCase):
    """Tests for the CaseCreator service."""

    fixtures = [
        "locations/countries.json",
        "locations/states.json",
        "locations/cities.json",
        "courts/courts.json",
    ]

    def setUp(self):
        self.creator = CaseCreator(extract_refs=False)
        self.court = Court.objects.exclude(pk=Court.DEFAULT_ID).first()

    def test_check_duplicate_returns_false_for_new_case(self):
        """Test duplicate check returns False for non-existent case."""
        result = self.creator.check_duplicate(self.court, "UNIQUE-FILE-123/99")
        self.assertFalse(result)

    def test_check_duplicate_returns_true_for_existing_case(self):
        """Test duplicate check returns True for existing case."""
        Case.objects.create(
            court=self.court,
            file_number="EXISTING-FILE-123/99",
            date=date(2021, 1, 1),
            content="<p>Test content</p>",
        )
        result = self.creator.check_duplicate(self.court, "EXISTING-FILE-123/99")
        self.assertTrue(result)

    def test_create_case_success(self):
        """Test successful case creation."""
        with patch.object(
            self.creator.court_resolver, "resolve", return_value=(self.court, None)
        ):
            case = self.creator.create_case(
                court_name=self.court.name,
                file_number="NEW-FILE-456/21",
                date=date(2021, 5, 15),
                content="<p>Case content</p>",
                case_type="Urteil",
            )

            self.assertIsNotNone(case.pk)
            self.assertEqual(case.court, self.court)
            self.assertEqual(case.file_number, "NEW-FILE-456/21")
            self.assertEqual(case.type, "Urteil")
            self.assertIsNotNone(case.slug)

    def test_create_case_duplicate_raises_error(self):
        """Test that creating duplicate case raises DuplicateCaseError."""
        # Create initial case
        Case.objects.create(
            court=self.court,
            file_number="DUP-FILE-789/21",
            date=date(2021, 1, 1),
            content="<p>Original content</p>",
        )

        with patch.object(
            self.creator.court_resolver, "resolve", return_value=(self.court, None)
        ):
            with self.assertRaises(DuplicateCaseError):
                self.creator.create_case(
                    court_name=self.court.name,
                    file_number="DUP-FILE-789/21",
                    date=date(2021, 5, 15),
                    content="<p>Duplicate content</p>",
                )

    def test_create_case_with_api_token_tracking(self):
        """Test that API token is tracked on created case."""
        user = User.objects.create_user(
            username="testuser", email="test@example.com", password="testpass123"
        )
        token = APIToken.objects.create(user=user, name="Test Token")

        with patch.object(
            self.creator.court_resolver, "resolve", return_value=(self.court, None)
        ):
            case = self.creator.create_case(
                court_name=self.court.name,
                file_number="TOKEN-FILE-111/21",
                date=date(2021, 5, 15),
                content="<p>Case content</p>",
                api_token=token,
            )

            self.assertEqual(case.created_by_token, token)
            # Cases created with API token are pending review (require approval)
            self.assertEqual(case.review_status, "pending")

    def test_create_case_sets_slug(self):
        """Test that slug is set correctly on created case."""
        with patch.object(
            self.creator.court_resolver, "resolve", return_value=(self.court, None)
        ):
            case = self.creator.create_case(
                court_name=self.court.name,
                file_number="SLUG-TEST-222/21",
                date=date(2021, 5, 15),
                content="<p>Case content</p>",
            )

            self.assertIsNotNone(case.slug)
            self.assertIn(self.court.slug, case.slug)
            self.assertIn("2021-05-15", case.slug)

    def test_create_case_default_source(self):
        """Test that default source is assigned when source_name is omitted."""
        with patch.object(
            self.creator.court_resolver, "resolve", return_value=(self.court, None)
        ):
            case = self.creator.create_case(
                court_name=self.court.name,
                file_number="SRC-DEFAULT-001/21",
                date=date(2021, 5, 15),
                content="<p>Case content</p>",
            )

            self.assertEqual(case.source_id, Source.DEFAULT_ID)

    def test_create_case_new_source_created(self):
        """Test that a new source is created when name does not exist."""
        with patch.object(
            self.creator.court_resolver, "resolve", return_value=(self.court, None)
        ):
            case = self.creator.create_case(
                court_name=self.court.name,
                file_number="SRC-NEW-002/21",
                date=date(2021, 5, 15),
                content="<p>Case content</p>",
                source_name="Brand New Source",
                source_homepage="https://newsource.example.com",
            )

            self.assertEqual(case.source.name, "Brand New Source")
            self.assertEqual(case.source.homepage, "https://newsource.example.com")
            self.assertTrue(Source.objects.filter(name="Brand New Source").exists())

    def test_create_case_existing_source_matched(self):
        """Test that an existing source is reused when name matches."""
        existing = Source.objects.create(
            name="Existing Source", homepage="https://existing.example.com"
        )
        with patch.object(
            self.creator.court_resolver, "resolve", return_value=(self.court, None)
        ):
            case = self.creator.create_case(
                court_name=self.court.name,
                file_number="SRC-EXIST-003/21",
                date=date(2021, 5, 15),
                content="<p>Case content</p>",
                source_name="Existing Source",
                source_homepage="https://different-url.example.com",
            )

            self.assertEqual(case.source_id, existing.pk)
            # Homepage should not be updated on existing source
            existing.refresh_from_db()
            self.assertEqual(existing.homepage, "https://existing.example.com")

    def test_resolve_source_creates_with_homepage(self):
        """Test resolve_source creates source with provided homepage."""
        source = self.creator.resolve_source("Test Source", "https://test.example.com")
        self.assertEqual(source.name, "Test Source")
        self.assertEqual(source.homepage, "https://test.example.com")

    def test_resolve_source_reuses_existing(self):
        """Test resolve_source returns existing source by name."""
        existing = Source.objects.create(
            name="Reuse Source", homepage="https://reuse.example.com"
        )
        source = self.creator.resolve_source(
            "Reuse Source", "https://other.example.com"
        )
        self.assertEqual(source.pk, existing.pk)


class CaseCreateSerializerTestCase(TestCase):
    """Tests for the CaseCreateSerializer."""

    def test_valid_data(self):
        """Test serializer with valid data."""
        data = {
            "court_name": "Bundesgerichtshof",
            "file_number": "I ZR 123/21",
            "date": "2021-05-15",
            "content": "<p>Case content with sufficient length</p>",
            "type": "Urteil",
        }
        serializer = CaseCreateSerializer(data=data)
        self.assertTrue(serializer.is_valid(), serializer.errors)

    def test_missing_required_field(self):
        """Test serializer rejects missing required fields."""
        data = {
            "court_name": "Bundesgerichtshof",
            # Missing file_number, date, content
        }
        serializer = CaseCreateSerializer(data=data)
        self.assertFalse(serializer.is_valid())
        self.assertIn("file_number", serializer.errors)
        self.assertIn("date", serializer.errors)
        self.assertIn("content", serializer.errors)

    def test_empty_court_name_rejected(self):
        """Test serializer rejects empty court name."""
        data = {
            "court_name": "",
            "file_number": "I ZR 123/21",
            "date": "2021-05-15",
            "content": "<p>Case content</p>",
        }
        serializer = CaseCreateSerializer(data=data)
        self.assertFalse(serializer.is_valid())
        self.assertIn("court_name", serializer.errors)

    def test_content_too_short(self):
        """Test serializer rejects content that is too short."""
        data = {
            "court_name": "Bundesgerichtshof",
            "file_number": "I ZR 123/21",
            "date": "2021-05-15",
            "content": "short",  # Less than 10 characters
        }
        serializer = CaseCreateSerializer(data=data)
        self.assertFalse(serializer.is_valid())
        self.assertIn("content", serializer.errors)

    @override_settings(CASE_CREATION_VALIDATION={"content_min_length": 5})
    def test_content_validation_uses_settings(self):
        """Test that content validation respects settings."""
        data = {
            "court_name": "Bundesgerichtshof",
            "file_number": "I ZR 123/21",
            "date": "2021-05-15",
            "content": "12345",  # Exactly 5 characters
        }
        serializer = CaseCreateSerializer(data=data)
        self.assertTrue(serializer.is_valid(), serializer.errors)

    def test_valid_data_with_source(self):
        """Test serializer accepts optional source field."""
        data = {
            "court_name": "Bundesgerichtshof",
            "file_number": "I ZR 123/21",
            "date": "2021-05-15",
            "content": "<p>Case content with sufficient length</p>",
            "source": {"name": "My Scraper", "homepage": "https://example.com"},
        }
        serializer = CaseCreateSerializer(data=data)
        self.assertTrue(serializer.is_valid(), serializer.errors)
        self.assertEqual(serializer.validated_data["source"]["name"], "My Scraper")

    def test_valid_data_with_source_name_only(self):
        """Test serializer accepts source with name only (no homepage)."""
        data = {
            "court_name": "Bundesgerichtshof",
            "file_number": "I ZR 123/21",
            "date": "2021-05-15",
            "content": "<p>Case content with sufficient length</p>",
            "source": {"name": "My Scraper"},
        }
        serializer = CaseCreateSerializer(data=data)
        self.assertTrue(serializer.is_valid(), serializer.errors)

    def test_source_empty_name_rejected(self):
        """Test serializer rejects source with empty name."""
        data = {
            "court_name": "Bundesgerichtshof",
            "file_number": "I ZR 123/21",
            "date": "2021-05-15",
            "content": "<p>Case content with sufficient length</p>",
            "source": {"name": ""},
        }
        serializer = CaseCreateSerializer(data=data)
        self.assertFalse(serializer.is_valid())
        self.assertIn("source", serializer.errors)

    def test_source_name_too_long_rejected(self):
        """Test serializer rejects source name exceeding 100 characters."""
        data = {
            "court_name": "Bundesgerichtshof",
            "file_number": "I ZR 123/21",
            "date": "2021-05-15",
            "content": "<p>Case content with sufficient length</p>",
            "source": {"name": "x" * 101},
        }
        serializer = CaseCreateSerializer(data=data)
        self.assertFalse(serializer.is_valid())
        self.assertIn("source", serializer.errors)

    def test_valid_data_without_source(self):
        """Test serializer is valid when source is omitted."""
        data = {
            "court_name": "Bundesgerichtshof",
            "file_number": "I ZR 123/21",
            "date": "2021-05-15",
            "content": "<p>Case content with sufficient length</p>",
        }
        serializer = CaseCreateSerializer(data=data)
        self.assertTrue(serializer.is_valid(), serializer.errors)
        self.assertNotIn("source", serializer.validated_data)


class CaseCreationAPITestCase(APITestCase):
    """Integration tests for the Case Creation API endpoint."""

    fixtures = [
        "locations/countries.json",
        "locations/states.json",
        "locations/cities.json",
        "courts/courts.json",
    ]

    def setUp(self):
        # Create user
        self.user = User.objects.create_user(
            username="testuser", email="test@example.com", password="testpass123"
        )

        # Create permission and permission group for write access
        self.write_permission, _ = APITokenPermission.objects.get_or_create(
            resource="cases", action="write"
        )
        self.permission_group = APITokenPermissionGroup.objects.create(
            name="test_write_group"
        )
        self.permission_group.permissions.add(self.write_permission)

        # Create API token with write permission
        self.token = APIToken.objects.create(
            user=self.user,
            name="Test Token",
            permission_group=self.permission_group,
        )

        self.client = APIClient()
        # Use force_authenticate to set both user and token (request.auth)
        self.client.force_authenticate(user=self.user, token=self.token)

        # Get a valid court for testing
        self.court = Court.objects.exclude(pk=Court.DEFAULT_ID).first()

    def _get_valid_case_data(self, file_number=None):
        """Helper to get valid case creation data."""
        return {
            "court_name": self.court.name,
            "file_number": file_number or "TEST-FILE-123/21",
            "date": "2021-05-15",
            "content": "<p>Test case content with sufficient length for validation</p>",
            "type": "Urteil",
        }

    @patch("oldp.apps.cases.services.case_creator.CaseCreator.create_case")
    def test_create_case_success(self, mock_create):
        """Test successful case creation returns 201 with id and slug."""
        mock_case = MagicMock()
        mock_case.id = 12345
        mock_case.slug = "test-court-2021-05-15-test-file-123-21"
        mock_create.return_value = mock_case

        data = self._get_valid_case_data()
        response = self.client.post("/api/cases/", data, format="json")

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertIn("id", response.data)
        self.assertIn("slug", response.data)

    @patch("oldp.apps.cases.services.case_creator.CaseCreator.create_case")
    def test_create_case_duplicate_returns_409(self, mock_create):
        """Test duplicate case returns 409 Conflict."""
        mock_create.side_effect = DuplicateCaseError(
            "A case with this court and file number already exists."
        )

        data = self._get_valid_case_data()
        response = self.client.post("/api/cases/", data, format="json")

        self.assertEqual(response.status_code, status.HTTP_409_CONFLICT)

    @patch("oldp.apps.cases.services.case_creator.CaseCreator.create_case")
    def test_create_case_court_not_found_returns_400(self, mock_create):
        """Test court not found returns 400 Bad Request."""
        mock_create.side_effect = CourtNotFoundError(
            "Could not resolve court from name."
        )

        data = self._get_valid_case_data()
        data["court_name"] = "Nonexistent Court XYZ 12345"
        response = self.client.post("/api/cases/", data, format="json")

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_create_case_missing_required_fields_returns_400(self):
        """Test missing required fields returns 400 Bad Request."""
        data = {"court_name": "Test Court"}  # Missing other required fields
        response = self.client.post("/api/cases/", data, format="json")

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertIn("file_number", response.data)
        self.assertIn("date", response.data)
        self.assertIn("content", response.data)

    def test_create_case_without_authentication_returns_401(self):
        """Test unauthenticated request returns 401."""
        # Clear authentication
        self.client.force_authenticate(user=None, token=None)
        data = self._get_valid_case_data()
        response = self.client.post("/api/cases/", data, format="json")

        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    def test_create_case_without_write_permission_returns_403(self):
        """Test request without write permission returns 403."""
        # Create token without write permission
        read_permission, _ = APITokenPermission.objects.get_or_create(
            resource="cases", action="read"
        )
        read_only_group = APITokenPermissionGroup.objects.create(name="read_only_group")
        read_only_group.permissions.add(read_permission)

        read_only_token = APIToken.objects.create(
            user=self.user, name="Read Only Token", permission_group=read_only_group
        )

        # Use force_authenticate with the read-only token
        self.client.force_authenticate(user=self.user, token=read_only_token)

        data = self._get_valid_case_data()
        response = self.client.post("/api/cases/", data, format="json")

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch("oldp.apps.cases.services.case_creator.CaseCreator.create_case")
    def test_create_case_tracks_api_token(self, mock_create):
        """Test that API token is passed to case creator."""
        mock_case = MagicMock()
        mock_case.id = 12345
        mock_case.slug = "test-slug"
        mock_create.return_value = mock_case

        data = self._get_valid_case_data()
        self.client.post("/api/cases/", data, format="json")

        # Verify create_case was called with api_token
        mock_create.assert_called_once()
        call_kwargs = mock_create.call_args[1]
        self.assertEqual(call_kwargs["api_token"], self.token)

    def test_create_case_content_too_short_returns_400(self):
        """Test content validation returns 400 for short content."""
        data = self._get_valid_case_data()
        data["content"] = "short"  # Less than minimum length
        response = self.client.post("/api/cases/", data, format="json")

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertIn("content", response.data)

    @patch("oldp.apps.cases.services.case_creator.CaseCreator.create_case")
    def test_create_case_with_all_optional_fields(self, mock_create):
        """Test case creation with all optional fields."""
        mock_case = MagicMock()
        mock_case.id = 12345
        mock_case.slug = "test-slug"
        mock_create.return_value = mock_case

        data = self._get_valid_case_data()
        data.update(
            {
                "ecli": "ECLI:DE:BGH:2021:150521UTEST123.21.0",
                "abstract": "<p>Case abstract summary</p>",
                "title": "Test Case Title",
            }
        )

        response = self.client.post("/api/cases/", data, format="json")

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        # Verify all fields were passed to create_case
        call_kwargs = mock_create.call_args[1]
        self.assertEqual(call_kwargs["ecli"], data["ecli"])
        self.assertEqual(call_kwargs["abstract"], data["abstract"])
        self.assertEqual(call_kwargs["title"], data["title"])

    @patch("oldp.apps.cases.services.case_creator.CaseCreator.create_case")
    def test_create_case_extract_refs_disabled_via_query_param(self, mock_create):
        """Test that extract_refs can be disabled via query parameter."""
        mock_case = MagicMock()
        mock_case.id = 12345
        mock_case.slug = "test-slug"
        mock_create.return_value = mock_case

        data = self._get_valid_case_data()

        # Use query parameter to disable extract_refs
        self.client.post("/api/cases/?extract_refs=false", data, format="json")

        call_kwargs = mock_create.call_args[1]
        self.assertFalse(call_kwargs["extract_refs"])

    @patch("oldp.apps.cases.services.case_creator.CaseCreator.create_case")
    def test_create_case_extract_refs_enabled_by_default(self, mock_create):
        """Test that extract_refs is enabled by default."""
        mock_case = MagicMock()
        mock_case.id = 12345
        mock_case.slug = "test-slug"
        mock_create.return_value = mock_case

        data = self._get_valid_case_data()

        self.client.post("/api/cases/", data, format="json")

        call_kwargs = mock_create.call_args[1]
        self.assertTrue(call_kwargs["extract_refs"])

    @patch("oldp.apps.cases.services.case_creator.CaseCreator.create_case")
    def test_create_case_with_source_passes_to_creator(self, mock_create):
        """Test that source data is passed through to case creator."""
        mock_case = MagicMock()
        mock_case.id = 12345
        mock_case.slug = "test-slug"
        mock_create.return_value = mock_case

        data = self._get_valid_case_data()
        data["source"] = {"name": "Test Source", "homepage": "https://test.example.com"}

        response = self.client.post("/api/cases/", data, format="json")

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        call_kwargs = mock_create.call_args[1]
        self.assertEqual(call_kwargs["source_name"], "Test Source")
        self.assertEqual(call_kwargs["source_homepage"], "https://test.example.com")

    @patch("oldp.apps.cases.services.case_creator.CaseCreator.create_case")
    def test_create_case_without_source_passes_none(self, mock_create):
        """Test that omitting source passes None to creator."""
        mock_case = MagicMock()
        mock_case.id = 12345
        mock_case.slug = "test-slug"
        mock_create.return_value = mock_case

        data = self._get_valid_case_data()
        response = self.client.post("/api/cases/", data, format="json")

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        call_kwargs = mock_create.call_args[1]
        self.assertIsNone(call_kwargs["source_name"])
        self.assertIsNone(call_kwargs["source_homepage"])

    @patch("oldp.apps.cases.services.case_creator.CaseCreator.create_case")
    def test_create_case_extract_refs_query_param_variations(self, mock_create):
        """Test various values for extract_refs query parameter."""
        mock_case = MagicMock()
        mock_case.id = 12345
        mock_case.slug = "test-slug"
        mock_create.return_value = mock_case

        data = self._get_valid_case_data()

        # Test "0" disables
        self.client.post("/api/cases/?extract_refs=0", data, format="json")
        self.assertFalse(mock_create.call_args[1]["extract_refs"])

        # Test "no" disables
        mock_create.reset_mock()
        self.client.post("/api/cases/?extract_refs=no", data, format="json")
        self.assertFalse(mock_create.call_args[1]["extract_refs"])

        # Test "true" enables
        mock_create.reset_mock()
        self.client.post("/api/cases/?extract_refs=true", data, format="json")
        self.assertTrue(mock_create.call_args[1]["extract_refs"])


class CaseCreationIntegrationTestCase(APITestCase):
    """Full integration tests for case creation without mocking.

    These tests verify the complete flow from API to database.
    """

    fixtures = [
        "locations/countries.json",
        "locations/states.json",
        "locations/cities.json",
        "courts/courts.json",
    ]

    def setUp(self):
        self.user = User.objects.create_user(
            username="integrationuser",
            email="integration@example.com",
            password="testpass",
        )

        write_permission, _ = APITokenPermission.objects.get_or_create(
            resource="cases", action="write"
        )
        permission_group = APITokenPermissionGroup.objects.create(
            name="integration_write_group"
        )
        permission_group.permissions.add(write_permission)

        self.token = APIToken.objects.create(
            user=self.user, name="Integration Token", permission_group=permission_group
        )

        self.client = APIClient()
        # Use force_authenticate to set both user and token (request.auth)
        self.client.force_authenticate(user=self.user, token=self.token)

        self.court = Court.objects.exclude(pk=Court.DEFAULT_ID).first()

    def test_full_case_creation_flow(self):
        """Test complete case creation from API to database."""
        data = {
            "court_name": self.court.name,
            "file_number": "INTEGRATION-TEST-999/21",
            "date": "2021-05-15",
            "content": "<p>Integration test case content with references to § 823 BGB</p>",
            "type": "Urteil",
        }

        # Use query param to disable ref extraction for faster test
        response = self.client.post(
            "/api/cases/?extract_refs=false", data, format="json"
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        # Verify case was created in database
        case_id = response.data["id"]
        case = Case.objects.get(pk=case_id)

        self.assertEqual(case.court, self.court)
        self.assertEqual(case.file_number, "INTEGRATION-TEST-999/21")
        self.assertEqual(str(case.date), "2021-05-15")
        self.assertEqual(case.type, "Urteil")
        self.assertEqual(case.created_by_token, self.token)
        self.assertIsNotNone(case.slug)
        # API-created cases require manual approval (review_status=pending)
        self.assertEqual(case.review_status, "pending")

    def test_api_created_cases_are_pending_by_default(self):
        """Test that cases created via API are always pending (require approval)."""
        data = {
            "court_name": self.court.name,
            "file_number": "PENDING-TEST-777/21",
            "date": "2021-05-15",
            "content": "<p>Test case content</p>",
        }

        response = self.client.post(
            "/api/cases/?extract_refs=false", data, format="json"
        )
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        # Verify case is pending review
        case = Case.objects.get(pk=response.data["id"])
        self.assertEqual(
            case.review_status,
            "pending",
            "API-created cases must be pending for approval workflow",
        )

    def test_duplicate_case_prevention(self):
        """Test that duplicate cases are prevented."""
        data = {
            "court_name": self.court.name,
            "file_number": "DUPLICATE-TEST-888/21",
            "date": "2021-05-15",
            "content": "<p>First case content</p>",
        }

        # Create first case (disable ref extraction for faster test)
        response1 = self.client.post(
            "/api/cases/?extract_refs=false", data, format="json"
        )
        self.assertEqual(response1.status_code, status.HTTP_201_CREATED)

        # Try to create duplicate
        response2 = self.client.post(
            "/api/cases/?extract_refs=false", data, format="json"
        )
        self.assertEqual(response2.status_code, status.HTTP_409_CONFLICT)

        # Verify only one case exists
        count = Case.objects.filter(
            court=self.court, file_number="DUPLICATE-TEST-888/21"
        ).count()
        self.assertEqual(count, 1)

    def test_case_creation_with_new_source(self):
        """Test that providing a new source name creates the source and assigns it."""
        data = {
            "court_name": self.court.name,
            "file_number": "SOURCE-NEW-001/21",
            "date": "2021-05-15",
            "content": "<p>Case content for source test</p>",
            "source": {
                "name": "Integration Test Source",
                "homepage": "https://integration.example.com",
            },
        }

        response = self.client.post(
            "/api/cases/?extract_refs=false", data, format="json"
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        case = Case.objects.get(pk=response.data["id"])
        self.assertEqual(case.source.name, "Integration Test Source")
        self.assertEqual(case.source.homepage, "https://integration.example.com")

    def test_case_creation_with_existing_source(self):
        """Test that providing an existing source name reuses it."""
        existing_source = Source.objects.create(
            name="Pre-Existing Source", homepage="https://preexisting.example.com"
        )

        data = {
            "court_name": self.court.name,
            "file_number": "SOURCE-EXIST-002/21",
            "date": "2021-05-15",
            "content": "<p>Case content for existing source test</p>",
            "source": {
                "name": "Pre-Existing Source",
                "homepage": "https://different.example.com",
            },
        }

        response = self.client.post(
            "/api/cases/?extract_refs=false", data, format="json"
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        case = Case.objects.get(pk=response.data["id"])
        self.assertEqual(case.source_id, existing_source.pk)
        # Homepage of existing source should not change
        existing_source.refresh_from_db()
        self.assertEqual(existing_source.homepage, "https://preexisting.example.com")

    def test_case_creation_without_source_uses_default(self):
        """Test that omitting source assigns the default source."""
        data = {
            "court_name": self.court.name,
            "file_number": "SOURCE-DEFAULT-003/21",
            "date": "2021-05-15",
            "content": "<p>Case content without source</p>",
        }

        response = self.client.post(
            "/api/cases/?extract_refs=false", data, format="json"
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        case = Case.objects.get(pk=response.data["id"])
        self.assertEqual(case.source_id, Source.DEFAULT_ID)



import logging

from allauth.account.models import EmailAddress
from django.contrib.auth.models import User
from django.core.management.base import BaseCommand, CommandError

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = "Load dummy users (admin and regular user) for testing"

    def handle(self, *args, **options):
        # Check if users already exist
        admin_exists = User.objects.filter(username="admin").exists()
        user_exists = User.objects.filter(username="user").exists()

        if admin_exists or user_exists:
            raise CommandError(
                "Dummy users already exist. Please delete them first if you want to reload."
            )

        # Create admin user
        admin = User.objects.create_superuser(
            username="admin", email="admin@example.com", password="admin"
        )
        # Get or create and verify email address for admin
        admin_email, created = EmailAddress.objects.get_or_create(
            user=admin,
            email="admin@example.com",
            defaults={"verified": True, "primary": True},
        )
        if not created:
            admin_email.verified = True
            admin_email.primary = True
            admin_email.save()
        logger.info("Created admin user (username: admin, password: admin)")

        # Create regular user
        user = User.objects.create_user(
            username="user", email="user@example.com", password="user"
        )
        # Get or create and verify email address for user
        user_email, created = EmailAddress.objects.get_or_create(
            user=user,
            email="user@example.com",
            defaults={"verified": True, "primary": True},
        )
        if not created:
            user_email.verified = True
            user_email.primary = True
            user_email.save()
        logger.info("Created user (username: user, password: user)")

        logger.info("Dummy users loaded successfully")



import json
import logging
import os
import shutil

from django.conf import settings
from django.core.management import BaseCommand
from django.core.paginator import Paginator

from oldp.api.urls import router

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    """Export data to JSONL (one JSON per line) using API serializers

    Usage: python manage.py dump_api_data ./workingdir/dumps

    Compress all dumps: gzip -r ./workingdir/dumps/*

    """

    help = "Export API data as JSON"
    chunk_size = 1000

    def __init__(self):
        super(Command, self).__init__()

    def add_arguments(self, parser):
        parser.add_argument(
            "output",
            type=str,
            help="Path relative to working directory ({})".format(settings.WORKING_DIR),
        )

        parser.add_argument(
            "--override",
            action="store_true",
            default=False,
            help="Override existing output files",
        )

        parser.add_argument(
            "--limit",
            type=int,
            default=0,
            help="Max. number of references per content type (default: 0, 0=unlimited)",
        )

    def handle(self, *args, **opts):
        dir_path = os.path.join(settings.WORKING_DIR, opts["output"])

        if os.path.exists(dir_path):
            if opts["override"]:
                shutil.rmtree(dir_path)
            else:
                logger.error("Output directory exist already: %s" % dir_path)
                return

        os.mkdir(dir_path)

        for api_register in router.registry:
            plural, view_set_cls, singular = api_register

            if "/" in plural or plural == "users":
                logger.debug("Skip non-root endpoints (and users): %s" % plural)
                continue

            file_path = os.path.join(dir_path, plural + ".jsonl")
            # view_set_cls = CaseViewSet
            view_set = view_set_cls()
            serializer_cls = view_set.get_serializer_class()
            # serializer = serializer_cls()
            qs = view_set.get_queryset()

            logger.debug("Writing to %s" % file_path)

            with open(file_path, "w") as file:
                # Use paginator to not load all rows at once in memory
                paginator = Paginator(qs, self.chunk_size)
                for page in range(1, paginator.num_pages + 1):
                    logger.debug(
                        "%s - total %i - page %i / %i"
                        % (plural, paginator.count, page, paginator.num_pages)
                    )

                    # Iterate over items and convert to JSON
                    for item in paginator.page(page).object_list:
                        data = serializer_cls(instance=item).data

                        # Append to file
                        file.write(json.dumps(data) + "\n")

        logger.info("Done")



import logging

from oldp.apps.laws.models import Law
from oldp.apps.laws.processing.processing_steps import LawProcessingStep

logger = logging.getLogger(__name__)


class ProcessingStep(LawProcessingStep):
    description = "Set review_status=accepted"

    def process(self, law: Law):
        law.review_status = "accepted"

        return law



from django.contrib import admin

from oldp.apps.topics.models import Topic


@admin.register(Topic)
class TopicAdmin(admin.ModelAdmin):
    date_hierarchy = "updated_date"
    list_display = ("title", "created_date", "updated_date")
    search_fields = ["title"]



from django.contrib import admin
from django.contrib.admin import SimpleListFilter

from oldp.apps.processing.admin import ProcessingStepActionsAdmin

from .models import City, Country, Court, State

admin.site.register(Country)
admin.site.register(State)
admin.site.register(City)


class APISubmissionFilter(SimpleListFilter):
    title = "API submission"
    parameter_name = "api_submission"

    def lookups(self, request, model_admin):
        return [
            ("yes", "Created via API"),
            ("no", "Not created via API"),
        ]

    def queryset(self, request, queryset):
        if self.value() == "yes":
            return queryset.exclude(created_by_token__isnull=True)
        if self.value() == "no":
            return queryset.filter(created_by_token__isnull=True)


@admin.register(Court)
class CourtAdmin(ProcessingStepActionsAdmin):
    ordering = ("name",)
    date_hierarchy = "updated_date"
    list_display = (
        "name",
        "slug",
        "court_type",
        "city",
        "code",
        "updated_date",
        "created_date",
        "created_by_token",
        "review_status",
    )
    list_filter = (
        APISubmissionFilter,
        "review_status",
        "created_by_token",
    )
    list_select_related = ("created_by_token",)
    actions = ["save_court"]
    search_fields = ["name", "slug", "code"]

    def lookup_allowed(self, lookup, value):
        if lookup == "created_date__date":
            return True
        return super().lookup_allowed(lookup, value)

    def save_court(self, request, queryset):
        for item in queryset:
            item.save()

    save_court.short_description = "Re-save"
